# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import torch
import torch.nn as nn
import hashlib
from PIL import Image
from dataclasses import dataclass
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from diffusers.utils import numpy_to_pil
from unitorch.cli import WriterMixin, WriterOutputs
from unitorch.cli import (
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli.models.modeling_utils import TensorsOutputs, TensorsTargets


@dataclass
class DiffusionOutputs(TensorsOutputs, WriterMixin):
    outputs: torch.Tensor


@dataclass
class DiffusionTargets(TensorsTargets):
    targets: torch.Tensor
    masks: Optional[torch.Tensor] = torch.empty(0)


class DiffusionProcessor:
    def __init__(self, image_folder: Optional[str] = None):
        assert image_folder is not None
        self.image_folder = image_folder
        if not os.path.exists(image_folder):
            os.makedirs(self.image_folder, exist_ok=True)

    @classmethod
    @add_default_section_for_init("core/process/diffusion")
    def from_core_configure(cls, config, **kwargs):
        pass

    def save(self, image: Image.Image):
        md5 = hashlib.md5()
        md5.update(image.tobytes())
        name = md5.hexdigest() + ".jpg"
        image.save(f"{self.image_folder}/{name}")
        return name

    @register_process("core/postprocess/diffusion")
    def _diffusion(
        self,
        outputs: DiffusionOutputs,
    ):
        results = outputs.to_pandas()
        assert results.shape[0] == 0 or results.shape[0] == outputs.outputs.shape[0]
        images = outputs.outputs.numpy()
        images = numpy_to_pil(images)
        results["diffusion"] = [self.save(image) for image in images]
        return WriterOutputs(results)


def diffusion_model_decorator(cls):
    class DiffusionModel(nn.Module):
        def __init__(self, *args, **kwargs):
            super().__init__()
            if "__diffusion_model__" in kwargs:
                self.model = kwargs.pop("__diffusion_model__")
            else:
                self.model = cls(*args, **kwargs)

            __more_attrs__ = [
                "load_state_dict",
                "state_dict",
                "save_checkpoint",
                "from_checkpoint",
                "from_pretrained",
            ]
            for __more_attr__ in __more_attrs__:
                setattr(self, __more_attr__, getattr(self.model, __more_attr__))

            self.model.register_forward_hook(self._hook)
            self.__in_training__ = False

        def _hook(self, module, inputs, outputs):
            self.__in_training__ = True

        def forward(self, *args, **kwargs):
            if self.training or self.__in_training__:
                return self.model(*args, **kwargs)
            return self.model.generate(*args, **kwargs)

        @classmethod
        def from_core_configure(_cls, cfg, **kwargs):
            model = cls.from_core_configure(cfg, **kwargs)
            return _cls(__diffusion_model__=model)

    return DiffusionModel



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
import numpy as np
import torch.nn.functional as F
import transformers
from itertools import accumulate
from collections import UserDict
from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
from transformers.generation.logits_process import _calc_banned_ngram_tokens
from unitorch.utils.decorators import replace
from unitorch.ops.ngram_repeat_block import NGramRepeatBlock


@replace(transformers.generation.logits_process.NoRepeatNGramLogitsProcessor)
class NoRepeatNGramLogitsProcessorV2(
    transformers.generation.logits_process.LogitsProcessor
):
    r"""
    Args:
        ngram_size (:obj:`int`):
            All ngrams of size :obj:`ngram_size` can only occur once.
    """

    def __init__(self, ngram_size: int):
        if not isinstance(ngram_size, int) or ngram_size <= 0:
            raise ValueError(
                f"`ngram_size` has to be a strictly positive integer, but is {ngram_size}"
            )
        self.ngram_size = ngram_size
        if torch.cuda.is_available() and NGramRepeatBlock.is_available():
            self.no_repeat_ngram_op = NGramRepeatBlock()
        else:
            self.no_repeat_ngram_op = None

    def __call__(
        self, input_ids: torch.Tensor, scores: torch.FloatTensor
    ) -> torch.FloatTensor:
        num_batch_hypotheses = scores.shape[0]
        cur_len = input_ids.shape[-1]
        if self.no_repeat_ngram_op and input_ids.device.type == "cuda":
            scores = self.no_repeat_ngram_op(
                input_ids,
                scores.float(),
                num_batch_hypotheses,
                cur_len - 1,
                1,
                self.ngram_size,
            )
        else:
            banned_batch_tokens = _calc_banned_ngram_tokens(
                self.ngram_size, input_ids, num_batch_hypotheses, cur_len
            )

            for i, banned_tokens in enumerate(banned_batch_tokens):
                scores[i, banned_tokens] = -float("inf")

        return scores


@replace(transformers.generation.beam_search.BeamSearchScorer)
class BeamSearchScorerV2(transformers.generation.beam_search.BeamSearchScorer):
    def __init__(
        self,
        batch_size: int,
        num_beams: int,
        device: torch.device,
        length_penalty: Optional[float] = 1.0,
        do_early_stopping: Optional[bool] = False,
        num_beam_hyps_to_keep: Optional[int] = 1,
        num_beam_groups: Optional[int] = 1,
        max_length: Optional[int] = None,
        use_reorder_cache_v2: Optional[bool] = False,
    ):
        super().__init__(
            batch_size,
            num_beams,
            device,
            length_penalty,
            do_early_stopping,
            num_beam_hyps_to_keep,
            num_beam_groups,
            max_length,
        )
        self.use_reorder_cache_v2 = use_reorder_cache_v2
        self.beams_offset = (
            (torch.arange(0, batch_size) * num_beams // self.num_beam_groups)
            .unsqueeze(1)
            .to(device)
        )
        self.cand_size = 2 * num_beams // self.num_beam_groups
        self.cand_offsets = torch.arange(0, self.cand_size).to(device)

    def process(
        self,
        input_ids: torch.Tensor,
        next_scores: torch.FloatTensor,
        next_tokens: torch.Tensor,
        next_indices: torch.Tensor,
        pad_token_id: Optional[int] = None,
        eos_token_id: Optional[Union[int, List[int]]] = None,
        beam_indices: Optional[torch.Tensor] = None,
    ) -> Tuple[torch.Tensor]:
        cur_len = input_ids.shape[-1]
        batch_size = len(self._beam_hyps)
        if not (batch_size == (input_ids.shape[0] // self.group_size)):
            if self.num_beam_groups > 1:
                raise ValueError(
                    f"A group beam size of {input_ids.shape[0]} is used as the input, but a group beam "
                    f"size of {self.group_size} is expected by the beam scorer."
                )
            else:
                raise ValueError(
                    f"A beam size of {input_ids.shape[0]} is used as the input, but a beam size of "
                    f"{self.group_size} is expected by the beam scorer."
                )

        next_tokens_id = next_tokens
        next_beams_id = next_indices
        effective_beam_id = next_beams_id + self.beams_offset

        if eos_token_id is not None:
            if isinstance(eos_token_id, int):
                eos_token_id = [eos_token_id]
            eos_mask = torch.zeros_like(next_tokens)
            for eos_token in eos_token_id:
                eos_mask = eos_mask + next_tokens.eq(eos_token)
            eos_mask = eos_mask.bool()
        else:
            eos_mask = torch.zeros_like(next_tokens).bool()
        eos_effective_idx = torch.masked_select(
            effective_beam_id[:, : self.group_size], mask=eos_mask[:, : self.group_size]
        )

        finished_batch_idxs = []
        if self.use_reorder_cache_v2 and eos_effective_idx.numel() > 0:
            eos_effective_scores = torch.masked_select(
                next_scores[:, : self.group_size], mask=eos_mask[:, : self.group_size]
            )
            input_clone = input_ids.index_select(0, eos_effective_idx)
            unfin_offset = np.array(list(accumulate(map(int, self._done))))[
                np.array(list(map(int, self._done))) == 0
            ]
            for i in range(eos_effective_idx.size(0)):
                eos_idx = eos_effective_idx[i]
                eos_score = eos_effective_scores[i]
                unfin_batch_idx = eos_idx // self.group_size
                batch_idx = unfin_batch_idx + unfin_offset[unfin_batch_idx]
                if beam_indices is not None:
                    batch_beam_idx = batch_idx * self.group_size + (
                        eos_idx % self.group_size
                    )
                    beam_index = beam_indices[batch_beam_idx]
                    beam_index = beam_index + (batch_beam_idx,)
                else:
                    beam_index = None
                if not self._done[batch_idx]:
                    self._beam_hyps[batch_idx.item()].add(
                        input_clone[i],
                        eos_score.item(),
                        beam_indices=beam_index,
                    )
                is_done = bool(self._done[batch_idx])
                self._done[batch_idx] = self._done[batch_idx] or self._beam_hyps[
                    batch_idx
                ].is_done(next_scores[unfin_batch_idx].max().item(), cur_len)
                if is_done != bool(self._done[batch_idx]):
                    finished_batch_idxs.append(unfin_batch_idx)

        if not self.use_reorder_cache_v2:
            eos_effective_scores = torch.masked_select(
                next_scores[:, : self.group_size], mask=eos_mask[:, : self.group_size]
            )
            input_ids_cpu = input_ids.cpu()
            eos_effective_idx_cpu = eos_effective_idx.cpu()
            eos_effective_scores_cpu = eos_effective_scores.cpu()
            for i in range(0, eos_effective_idx_cpu.size()[-1]):
                eos_idx = eos_effective_idx_cpu[i]
                batch_idx = eos_idx // self.group_size
                if beam_indices is not None:
                    batch_beam_idx = batch_idx * self.group_size + (
                        eos_idx % self.group_size
                    )
                    beam_index = beam_indices[batch_beam_idx]
                    beam_index = beam_index + (batch_beam_idx,)
                else:
                    beam_index = None
                if not self._done[batch_idx]:
                    self._beam_hyps[batch_idx.item()].add(
                        input_ids_cpu[eos_idx].clone(),
                        eos_effective_scores_cpu[i],
                        beam_indices=beam_index,
                    )
                self._done[batch_idx] = self._done[batch_idx] or self._beam_hyps[
                    batch_idx
                ].is_done(next_scores[batch_idx].max().item(), cur_len)

        if self.use_reorder_cache_v2 and len(finished_batch_idxs) > 0:
            new_batch_size = batch_size - len(finished_batch_idxs)
            batch_mask = torch.ones(batch_size).to(next_tokens_id)
            batch_mask[torch.tensor(finished_batch_idxs)] = 0
            batch_idxs = batch_mask.nonzero(as_tuple=False).squeeze(-1)
            eos_mask = eos_mask[batch_idxs]
            next_beams_id = next_beams_id[batch_idxs]
            self.beams_offset.resize_(new_batch_size, 1)
            effective_beam_id = next_beams_id.add(self.beams_offset)
            next_scores = next_scores[batch_idxs]
            next_tokens = next_tokens[batch_idxs]
            next_tokens_id = next_tokens_id[batch_idxs]
            before_batch_size = batch_size
            batch_size = new_batch_size
        else:
            before_batch_size = batch_size
            batch_idxs = None

        active_mask = torch.add(
            eos_mask.type_as(self.cand_offsets) * self.cand_size,
            self.cand_offsets[: eos_mask.size(1)],
        )
        _, active_hypos = torch.topk(
            active_mask, k=self.group_size, dim=1, largest=False
        )
        active_effective_beam_id = torch.gather(
            effective_beam_id, dim=1, index=active_hypos
        )
        active_scores = torch.gather(next_scores, dim=1, index=active_hypos)
        active_tokens = torch.gather(next_tokens_id, dim=1, index=active_hypos)
        beam_idxs = active_effective_beam_id.view(-1)
        beam_scores = active_scores.view(-1)
        beam_tokens = active_tokens.view(-1)

        if batch_idxs is not None:
            new_beam_idxs = (
                torch.arange(before_batch_size * self.group_size)
                .reshape(before_batch_size, self.group_size)
                .to(input_ids)
            )
            beam_idxs = new_beam_idxs[batch_idxs].reshape(-1)[beam_idxs]

        userdict = UserDict(
            {
                "next_beam_scores": beam_scores.view(-1),
                "next_beam_tokens": beam_tokens.view(-1),
                "next_beam_indices": beam_idxs.view(-1),
            }
        )
        if self.use_reorder_cache_v2:
            userdict["next_batch_indices"] = batch_idxs

        return userdict

    def finalize(
        self,
        input_ids: torch.Tensor,
        final_beam_scores: torch.FloatTensor,
        final_beam_tokens: torch.Tensor,
        final_beam_indices: torch.Tensor,
        max_length: int,
        pad_token_id: Optional[int] = None,
        eos_token_id: Optional[Union[int, List[int]]] = None,
        beam_indices: Optional[torch.Tensor] = None,
    ) -> Tuple[torch.Tensor]:
        batch_size = len(self._beam_hyps)

        unfin_offset = np.array(list(accumulate(map(int, self._done))))[
            np.array(list(map(int, self._done))) == 0
        ]
        if self.use_reorder_cache_v2:
            batch_size = len(unfin_offset)
        for batch_idx in range(batch_size):
            if not self.use_reorder_cache_v2 and self._done[batch_idx]:
                continue
            if self.use_reorder_cache_v2:
                final_batch_idx = batch_idx + unfin_offset[batch_idx]
            else:
                final_batch_idx = batch_idx
            # need to add best num_beams hypotheses to generated hyps
            for beam_id in range(self.num_beams):
                effective_beam_id = batch_idx * self.num_beams + beam_id
                final_score = final_beam_scores[effective_beam_id].item()
                final_tokens = input_ids[effective_beam_id]
                beam_index = (
                    beam_indices[effective_beam_id]
                    if beam_indices is not None
                    else None
                )
                self._beam_hyps[final_batch_idx].add(
                    final_tokens, final_score, beam_indices=beam_index
                )

        batch_size = len(self._beam_hyps)

        # select the best hypotheses
        sent_lengths = input_ids.new(batch_size * self.num_beam_hyps_to_keep)
        best = []
        best_indices = []
        best_scores = torch.zeros(
            batch_size * self.num_beam_hyps_to_keep,
            device=self.device,
            dtype=torch.float32,
        )

        # retrieve best hypotheses
        for i, beam_hyp in enumerate(self._beam_hyps):
            sorted_hyps = sorted(beam_hyp.beams, key=lambda x: x[0])
            for j in range(self.num_beam_hyps_to_keep):
                best_hyp_tuple = sorted_hyps.pop()
                best_score = best_hyp_tuple[0]
                best_hyp = best_hyp_tuple[1]
                best_index = best_hyp_tuple[2]
                sent_lengths[self.num_beam_hyps_to_keep * i + j] = len(best_hyp)

                # append to lists
                best.append(best_hyp)
                best_indices.append(best_index)
                best_scores[i * self.num_beam_hyps_to_keep + j] = best_score

        # prepare for adding eos
        sent_lengths_max = sent_lengths.max().item() + 1
        sent_max_len = (
            min(sent_lengths_max, max_length)
            if max_length is not None
            else sent_lengths_max
        )
        decoded: torch.Tensor = input_ids.new(
            batch_size * self.num_beam_hyps_to_keep, sent_max_len
        )
        if len(best_indices) > 0 and best_indices[0] is not None:
            indices: torch.Tensor = input_ids.new(
                batch_size * self.num_beam_hyps_to_keep, sent_max_len
            )
        else:
            indices = None
        # shorter batches are padded if needed
        if sent_lengths.min().item() != sent_lengths.max().item():
            assert pad_token_id is not None, "`pad_token_id` has to be defined"
            decoded.fill_(pad_token_id)

        if indices is not None:
            indices.fill_(-1)

        # fill with hypotheses and eos_token_id if the latter fits in
        for i, (hypo, best_idx) in enumerate(zip(best, best_indices)):
            decoded[i, : sent_lengths[i]] = hypo
            if indices is not None:
                indices[i, : len(best_idx)] = torch.tensor(best_idx)
            if sent_lengths[i] < sent_max_len:
                decoded[i, sent_lengths[i]] = (
                    eos_token_id if isinstance(eos_token_id, int) else eos_token_id[0]
                )
        return UserDict(
            {
                "sequences": decoded,
                "sequence_scores": best_scores,
                "beam_indices": indices,
            }
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

# pretrained infos
pretrained_chatglm_infos = {
    # chatglm
    "default-chatglm": {
        "config": "https://huggingface.co/THUDM/chatglm-6b/resolve/main/config.json",
        "vocab": "https://huggingface.co/THUDM/chatglm-6b/resolve/main/ice_text.model",
        "tokenizer": "https://huggingface.co/THUDM/chatglm-6b/raw/main/tokenizer_config.json",
    },
    "chatglm-6b": {
        "config": "https://huggingface.co/THUDM/chatglm-6b/resolve/main/config.json",
        "vocab": "https://huggingface.co/THUDM/chatglm-6b/resolve/main/ice_text.model",
        "tokenizer": "https://huggingface.co/THUDM/chatglm-6b/raw/main/tokenizer_config.json",
        "weight": [
            f"https://huggingface.co/THUDM/chatglm-6b/resolve/main/pytorch_model-{str(i).rjust(5, '0')}-of-00008.bin"
            for i in range(1, 9)
        ],
    },
}

import unitorch.cli.models.chatglm.modeling
import unitorch.cli.models.chatglm.processing
from unitorch.cli.models.chatglm.modeling import (
    ChatGLMForClassification,
    ChatGLMForPretrain,
    ChatGLMForGeneration,
)
from unitorch.cli.models.chatglm.processing import ChatGLMProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import logging
import torch
import torch.nn as nn
import torch.nn.functional as F
import transformers
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers import PegasusXConfig, PegasusModel, PegasusXForConditionalGeneration
from transformers.models.pegasus_x.modeling_pegasus_x import (
    PegasusXEncoder,
    PegasusXDecoder,
)

from unitorch.utils.decorators import replace
from unitorch.models import GenericModel, GenericOutputs


@replace(transformers.models.pegasus_x.modeling_pegasus_x.PegasusXAttention)
class PegasusXAttentionV2(
    transformers.models.pegasus_x.modeling_pegasus_x.PegasusXAttention
):
    def __init__(
        self,
        embed_dim: int,
        num_heads: int,
        dropout: Optional[float] = 0.0,
        is_decoder: Optional[bool] = False,
        bias: Optional[bool] = True,
    ):
        super().__init__(
            embed_dim=embed_dim,
            num_heads=num_heads,
            dropout=dropout,
            is_decoder=is_decoder,
            bias=bias,
        )

    def forward(
        self,
        hidden_states: torch.Tensor,
        key_value_states: Optional[torch.Tensor] = None,
        past_key_value: Optional[Tuple[torch.Tensor]] = None,
        attention_mask: Optional[torch.Tensor] = None,
        layer_head_mask: Optional[torch.Tensor] = None,
        output_attentions: Optional[bool] = False,
    ) -> Tuple[torch.Tensor, Optional[torch.Tensor], Optional[Tuple[torch.Tensor]]]:
        """Input shape: Batch x Time x Channel"""

        # if key_value_states are provided this layer is used as a cross-attention layer
        # for the decoder
        is_cross_attention = key_value_states is not None
        bsz, tgt_len, embed_dim = hidden_states.size()
        kv_bsz = bsz

        # get query proj
        query_states = self.q_proj(hidden_states) * self.scaling
        # get key, value proj
        if is_cross_attention and past_key_value is not None:
            # reuse k,v, cross_attentions
            key_states = past_key_value[0]
            value_states = past_key_value[1]
            kv_bsz = key_states.size(0)
        elif is_cross_attention:
            # cross_attentions
            kv_bsz = key_value_states.size(0)
            key_states = self._shape(self.k_proj(key_value_states), -1, kv_bsz)
            value_states = self._shape(self.v_proj(key_value_states), -1, kv_bsz)
        elif past_key_value is not None:
            # reuse k, v, self_attention
            key_states = self._shape(self.k_proj(hidden_states), -1, bsz)
            value_states = self._shape(self.v_proj(hidden_states), -1, bsz)
            key_states = torch.cat([past_key_value[0], key_states], dim=2)
            value_states = torch.cat([past_key_value[1], value_states], dim=2)
        else:
            # self_attention
            key_states = self._shape(self.k_proj(hidden_states), -1, bsz)
            value_states = self._shape(self.v_proj(hidden_states), -1, bsz)

        if self.is_decoder:
            # if cross_attention save Tuple(torch.Tensor, torch.Tensor) of all cross attention key/value_states.
            # Further calls to cross_attention layer can then reuse all cross-attention
            # key/value_states (first "if" case)
            # if uni-directional self-attention (decoder) save Tuple(torch.Tensor, torch.Tensor) of
            # all previous decoder key/value_states. Further calls to uni-directional self-attention
            # can concat previous decoder key/value_states to current projected key/value_states (third "elif" case)
            # if encoder bi-directional self-attention `past_key_value` is always `None`
            past_key_value = (key_states, value_states)

        proj_shape = (bsz * self.num_heads, -1, self.head_dim)
        query_states = self._shape(query_states, tgt_len, bsz).view(*proj_shape)
        kv_proj_shape = (kv_bsz * self.num_heads, -1, self.head_dim)
        key_states = key_states.view(*kv_proj_shape)
        value_states = value_states.view(*kv_proj_shape)

        src_len = key_states.size(1)

        if is_cross_attention and kv_bsz != bsz:
            attn_weights = torch.einsum(
                "bxhtd,bhsd->bxhts",
                query_states.view(kv_bsz, -1, self.num_heads, *query_states.size()[1:]),
                key_states.view(kv_bsz, self.num_heads, *key_states.size()[1:]),
            )
            attn_weights = attn_weights.reshape(-1, *attn_weights.size()[-2:])
        else:
            attn_weights = torch.bmm(query_states, key_states.transpose(1, 2))

        assert attn_weights.size() == (
            bsz * self.num_heads,
            tgt_len,
            src_len,
        ), f"Attention weights should be of size {(bsz * self.num_heads, tgt_len, src_len)}, but is {attn_weights.size()}"

        if attention_mask is not None:
            assert attention_mask.size() == (
                bsz,
                1,
                tgt_len,
                src_len,
            ), f"Attention mask should be of size {(bsz, 1, tgt_len, src_len)}, but is {attention_mask.size()}"
            attn_weights = (
                attn_weights.view(bsz, self.num_heads, tgt_len, src_len)
                + attention_mask
            )
            attn_weights = attn_weights.view(bsz * self.num_heads, tgt_len, src_len)

        attn_weights = F.softmax(attn_weights, dim=-1)

        if layer_head_mask is not None:
            assert layer_head_mask.size() == (
                self.num_heads,
            ), f"Head mask for a single layer should be of size {(self.num_heads,)}, but is {layer_head_mask.size()}"
            attn_weights = layer_head_mask.view(1, -1, 1, 1) * attn_weights.view(
                bsz, self.num_heads, tgt_len, src_len
            )
            attn_weights = attn_weights.view(bsz * self.num_heads, tgt_len, src_len)

        if output_attentions:
            # this operation is a bit akward, but it's required to
            # make sure that attn_weights keeps its gradient.
            # In order to do so, attn_weights have to reshaped
            # twice and have to be reused in the following
            attn_weights_reshaped = attn_weights.view(
                bsz, self.num_heads, tgt_len, src_len
            )
            attn_weights = attn_weights_reshaped.view(
                bsz * self.num_heads, tgt_len, src_len
            )
        else:
            attn_weights_reshaped = None

        attn_probs = F.dropout(attn_weights, p=self.dropout, training=self.training)

        if is_cross_attention and bsz != kv_bsz:
            attn_probs = attn_probs.to(value_states.dtype)
            attn_output = torch.einsum(
                "bxhts,bhsd->bxhtd",
                attn_probs.view(kv_bsz, -1, self.num_heads, *attn_probs.size()[1:]),
                value_states.view(kv_bsz, self.num_heads, *value_states.size()[1:]),
            )
            attn_output = attn_output.reshape(-1, *attn_output.size()[-2:])
        else:
            attn_probs = attn_probs.to(value_states.dtype)
            attn_output = torch.bmm(attn_probs, value_states)

        assert attn_output.size() == (
            bsz * self.num_heads,
            tgt_len,
            self.head_dim,
        ), f"`attn_output` should be of size {(bsz, self.num_heads, tgt_len, self.head_dim)}, but is {attn_output.size()}"

        attn_output = (
            attn_output.view(bsz, self.num_heads, tgt_len, self.head_dim)
            .transpose(1, 2)
            .reshape(bsz, tgt_len, embed_dim)
        )

        attn_output = self.out_proj(attn_output)

        return attn_output, attn_weights_reshaped, past_key_value


class XPegasusForGeneration(GenericModel):
    """
    XPegasus model for text generation tasks.
    """

    prefix_keys_in_state_dict = {
        "^(?!model\.model\.|model\.lm_head\.)model\.": "model.",
        "^lm_head.": "model.",
    }

    def __init__(
        self,
        config_path: str,
        gradient_checkpointing: Optional[bool] = False,
    ):
        """
        Initializes the XPegasusForGeneration model.

        Args:
            config_path (str): Path to the model configuration file.
            gradient_checkpointing (Optional[bool]): Whether to use gradient checkpointing. Defaults to False.
        """
        super().__init__()
        self.config = PegasusXConfig.from_json_file(config_path)
        self.config.gradient_checkpointing = gradient_checkpointing
        self.model = PegasusXForConditionalGeneration(self.config)
        self.init_weights()

    def forward(
        self,
        input_ids: torch.Tensor,
        attention_mask: torch.Tensor,
        decoder_input_ids: torch.Tensor,
        decoder_attention_mask: torch.Tensor,
    ):
        """
        Performs forward pass of the XPegasusForGeneration model.

        Args:
            input_ids (torch.Tensor): Tensor of input token IDs.
            attention_mask (torch.Tensor): Tensor of attention mask.
            decoder_input_ids (torch.Tensor): Tensor of decoder input token IDs.
            decoder_attention_mask (torch.Tensor): Tensor of decoder attention mask.

        Returns:
            (torch.Tensor):The model's logits.
        """
        outputs = self.model(
            input_ids=input_ids,
            attention_mask=attention_mask,
            decoder_input_ids=decoder_input_ids,
            decoder_attention_mask=decoder_attention_mask,
            return_dict=True,
        )
        logits = outputs.logits
        return logits

    @torch.no_grad()
    def generate(
        self,
        input_ids: torch.Tensor,
        num_beams: Optional[int] = 5,
        decoder_start_token_id: Optional[int] = 0,
        decoder_end_token_id: Optional[int] = 1,
        num_return_sequences: Optional[int] = 1,
        min_gen_seq_length: Optional[int] = 0,
        max_gen_seq_length: Optional[int] = 48,
        repetition_penalty: Optional[float] = 1.0,
        no_repeat_ngram_size: Optional[int] = 0,
        early_stopping: Optional[bool] = True,
        length_penalty: Optional[float] = 1.0,
        num_beam_groups: Optional[int] = 1,
        diversity_penalty: Optional[float] = 0.0,
        do_sample: Optional[bool] = False,
        temperature: Optional[float] = 1.0,
        top_k: Optional[int] = 50,
        top_p: Optional[float] = 1.0,
    ):
        """
        Generates sequences using the XPegasusForGeneration model.

        Args:
            input_ids: The input token IDs.
            num_beams (int, optional): The number of beams for beam search. Defaults to 5.
            decoder_start_token_id (int, optional): The decoder's start token ID. Defaults to 2.
            decoder_end_token_id (int, optional): The decoder's end token ID. Defaults to 2.
            num_return_sequences (int, optional): The number of generated sequences to return. Defaults to 1.
            min_gen_seq_length (int, optional): The minimum length of the generated sequences. Defaults to 0.
            max_gen_seq_length (int, optional): The maximum length of the generated sequences. Defaults to 48.
            repetition_penalty (float, optional): The repetition penalty. Defaults to 1.0.
            no_repeat_ngram_size (int, optional): The size of n-grams to avoid repeating. Defaults to 0.
            early_stopping (bool, optional): Whether to stop generation early. Defaults to True.
            length_penalty (float, optional): The length penalty. Defaults to 1.0.
            num_beam_groups (int, optional): The number of beam groups for diverse beam search. Defaults to 1.
            diversity_penalty (float, optional): The diversity penalty. Defaults to 0.0.
            do_sample (bool, optional): Whether to use sampling for generation. Defaults to False.
            temperature (float, optional): The temperature for sampling. Defaults to 1.0.
            top_k (int, optional): The value for top-k sampling. Defaults to 50.
            top_p (float, optional): The value for top-p (nucleus) sampling. Defaults to 1.0.

        Returns:
            GenericOutputs: The generated sequences and their scores.
        """
        outputs = self.model.generate(
            input_ids,
            max_length=max_gen_seq_length,
            min_length=min_gen_seq_length,
            num_beams=num_beams,
            do_sample=do_sample,
            decoder_start_token_id=decoder_start_token_id,
            no_repeat_ngram_size=no_repeat_ngram_size,
            early_stopping=early_stopping,
            length_penalty=length_penalty,
            repetition_penalty=repetition_penalty,
            num_return_sequences=num_return_sequences,
            bos_token_id=decoder_start_token_id,
            eos_token_id=decoder_end_token_id,
            num_beam_groups=num_beam_groups,
            diversity_penalty=diversity_penalty,
            temperature=temperature,
            top_k=top_k,
            top_p=top_p,
            return_dict_in_generate=True,
            output_scores=True,
        )

        sequences = outputs.sequences.reshape(
            -1, num_return_sequences, outputs.sequences.size(-1)
        )
        outputs.sequences = torch.zeros(
            sequences.size(0), num_return_sequences, max_gen_seq_length
        ).to(device=sequences.device)
        outputs.sequences[:, :, : sequences.size(-1)].copy_(sequences)

        if num_return_sequences == 1:
            outputs.sequences = outputs.sequences.reshape(-1, max_gen_seq_length)

        return GenericOutputs(
            sequences=outputs.sequences,
            sequences_scores=outputs.sequences_scores,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import torch
import numpy as np
from PIL import Image
from transformers import CLIPTokenizer
from torchvision.transforms import (
    Resize,
    CenterCrop,
    ToTensor,
    Normalize,
    Compose,
    RandomHorizontalFlip,
)
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.models import HfTextClassificationProcessor, GenericOutputs


class ControlNetProcessor(HfTextClassificationProcessor):
    def __init__(
        self,
        vocab_path: str,
        merge_path: str,
        max_seq_length: Optional[int] = 128,
        position_start_id: Optional[int] = 0,
        pixel_mean: List[float] = [0.5],
        pixel_std: List[float] = [0.5],
        resize_shape: Optional[List[int]] = [224, 224],
        crop_shape: Optional[List[int]] = [224, 224],
    ):
        tokenizer = CLIPTokenizer(
            vocab_file=vocab_path,
            merges_file=merge_path,
        )
        tokenizer.cls_token = tokenizer.bos_token
        tokenizer.sep_token = tokenizer.eos_token
        super().__init__(
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            position_start_id=position_start_id,
        )

        self.pixel_mean = torch.tensor(pixel_mean)
        self.pixel_std = torch.tensor(pixel_std)
        self.resize_shape = resize_shape
        self.crop_shape = crop_shape
        self.image_processor = Compose(
            [
                Resize(self.resize_shape),
                CenterCrop(self.crop_shape),
                ToTensor(),
                Normalize(self.pixel_mean, self.pixel_std),
            ]
        )
        self.condition_image_processor = Compose(
            [
                Resize(self.resize_shape),
                CenterCrop(self.crop_shape),
                ToTensor(),
            ]
        )

    def controlnet(
        self,
        prompt: str,
        image: Image.Image,
        condition_image: Image.Image,
        max_seq_length: Optional[int] = None,
    ):
        prompt_outputs = self.classification(prompt, max_seq_length=max_seq_length)
        pixel_values = self.image_processor(image)
        condition_pixel_values = self.condition_image_processor(condition_image)
        return GenericOutputs(
            input_ids=prompt_outputs.input_ids,
            attention_mask=prompt_outputs.attention_mask,
            pixel_values=pixel_values,
            condition_pixel_values=condition_pixel_values,
        )

    def controlnet_inputs(
        self,
        prompt: str,
        condition_image: Image.Image,
        negative_prompt: Optional[str] = "",
        max_seq_length: Optional[int] = None,
    ):
        prompt_outputs = self.classification(prompt, max_seq_length=max_seq_length)
        negative_prompt_outputs = self.classification(
            negative_prompt, max_seq_length=max_seq_length
        )
        return GenericOutputs(
            input_ids=prompt_outputs.input_ids,
            attention_mask=prompt_outputs.attention_mask,
            negative_input_ids=negative_prompt_outputs.input_ids,
            negative_attention_mask=negative_prompt_outputs.attention_mask,
            condition_pixel_values=self.condition_image_processor(condition_image),
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import json
import torch
import torch.nn.functional as F
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers import CLIPTextConfig, CLIPTextModel
from diffusers import EulerDiscreteScheduler, DPMSolverMultistepScheduler
from diffusers.models import (
    UNet2DModel,
    UNet2DConditionModel,
    AutoencoderKL,
)
from diffusers.schedulers import (
    DDPMScheduler,
    PNDMScheduler,
)
from diffusers.pipelines import (
    DDPMPipeline,
    StableDiffusionPipeline,
    StableDiffusionUpscalePipeline,
    StableDiffusionInpaintPipeline,
    StableDiffusionDepth2ImgPipeline,
)
from unitorch.models import GenericModel, GenericOutputs


class StableForImageGeneration(GenericModel):
    prefix_keys_in_state_dict = {
        # unet weights
        "^conv_in.*": "unet.",
        "^conv_norm_out.*": "unet.",
        "^conv_out.*": "unet.",
        "^time_embedding.*": "unet.",
        "^up_blocks.*": "unet.",
        "^mid_block.*": "unet.",
        "^down_blocks.*": "unet.",
        # vae weights
        "^encoder.*": "vae.",
        "^decoder.*": "vae.",
        "^post_quant_conv.*": "vae.",
        "^quant_conv.*": "vae.",
    }

    def __init__(
        self,
        config_path: str,
        scheduler_config_path: str,
        image_size: Optional[int] = 224,
        in_channels: Optional[int] = 3,
        out_channels: Optional[int] = 3,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        seed: Optional[int] = 1123,
    ):
        super().__init__()
        self.seed = seed
        self.num_train_timesteps = num_train_timesteps
        self.num_infer_timesteps = num_infer_timesteps

        config_dict = json.load(open(config_path))
        config_dict.update(
            {
                "sample_size": image_size,
                "in_channels": in_channels,
                "out_channels": out_channels,
            }
        )
        self.unet = UNet2DModel.from_config(config_dict)

        scheduler_config_dict = json.load(open(scheduler_config_path))
        self.scheduler = DDPMScheduler.from_config(scheduler_config_dict)

    def forward(self, pixel_values: torch.Tensor):
        noise = torch.randn(pixel_values.shape).to(pixel_values.device)
        batch = pixel_values.size(0)

        timesteps = torch.randint(
            0,
            self.scheduler.num_train_timesteps,
            (batch,),
            device=pixel_values.device,
        ).long()

        noise_pixel_values = self.scheduler.add_noise(
            pixel_values,
            noise,
            timesteps,
        )
        outputs = self.unet(
            noise_pixel_values,
            timesteps,
            return_dict=False,
        )[0]
        loss = F.mse_loss(outputs, noise)
        return loss

    def generate(self, batch_size: int):
        self.scheduler.set_timesteps(num_inference_steps=self.num_infer_timesteps)
        pipeline = DDPMPipeline(self.unet, self.scheduler, batch_size)
        images = pipeline(
            batch_size=batch_size,
            generator=torch.Generator(device=pipeline.device).manual_seed(self.seed),
        ).images

        return GenericOutputs(images=images)


class StableForText2ImageGeneration(GenericModel):
    prefix_keys_in_state_dict = {
        # unet weights
        "^conv_in.*": "unet.",
        "^conv_norm_out.*": "unet.",
        "^conv_out.*": "unet.",
        "^time_embedding.*": "unet.",
        "^up_blocks.*": "unet.",
        "^mid_block.*": "unet.",
        "^down_blocks.*": "unet.",
        # text weights
        "^text_model.*": "text.",
        # vae weights
        "^encoder.*": "vae.",
        "^decoder.*": "vae.",
        "^post_quant_conv.*": "vae.",
        "^quant_conv.*": "vae.",
    }

    replace_keys_in_state_dict = {
        "\.query\.": ".to_q.",
        "\.key\.": ".to_k.",
        "\.value\.": ".to_v.",
        "\.proj_attn\.": ".to_out.0.",
    }

    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        image_size: Optional[int] = 224,
        in_channels: Optional[int] = 4,
        out_channels: Optional[int] = 4,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        freeze_vae_encoder: Optional[bool] = True,
        freeze_text_encoder: Optional[bool] = True,
        seed: Optional[int] = 1123,
    ):
        super().__init__()
        self.seed = seed
        self.num_train_timesteps = num_train_timesteps
        self.num_infer_timesteps = num_infer_timesteps
        self.image_size = image_size

        config_dict = json.load(open(config_path))
        config_dict.update(
            {
                "sample_size": image_size,
                "in_channels": in_channels,
                "out_channels": out_channels,
            }
        )
        self.unet = UNet2DConditionModel.from_config(config_dict)

        text_config = CLIPTextConfig.from_json_file(text_config_path)
        self.text = CLIPTextModel(text_config)

        vae_config_dict = json.load(open(vae_config_path))
        self.vae = AutoencoderKL.from_config(vae_config_dict)

        scheduler_config_dict = json.load(open(scheduler_config_path))
        self.scheduler = DDPMScheduler.from_config(scheduler_config_dict)

        self.vae_scale_factor = 2 ** (len(self.vae.config.block_out_channels) - 1)

        if freeze_vae_encoder:
            self.vae.requires_grad = False

        if freeze_text_encoder:
            self.text.requires_grad = False

    def forward(
        self,
        pixel_values: torch.Tensor,
        input_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
    ):
        latents = self.vae.encode(pixel_values).latent_dist.sample()
        latents = latents * self.vae.config.scaling_factor
        noise = torch.randn(latents.shape).to(latents.device)
        batch = latents.size(0)

        timesteps = torch.randint(
            0,
            self.scheduler.num_train_timesteps,
            (batch,),
            device=pixel_values.device,
        ).long()

        noise_latents = self.scheduler.add_noise(
            latents,
            noise,
            timesteps,
        )

        encoder_hidden_states = self.text(input_ids, attention_mask)[0]
        outputs = self.unet(
            noise_latents,
            timesteps,
            encoder_hidden_states,
        ).sample
        if self.scheduler.config.prediction_type == "v_prediction":
            noise = self.scheduler.get_velocity(latents, noise, timesteps)
        loss = F.mse_loss(outputs, noise)
        return loss

    def generate(
        self,
        input_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        guidance_scale: Optional[float] = 7.5,
    ):
        prompt_embeds = self.text(
            input_ids,
            attention_mask,
        )[0]
        negative_prompt_embeds = self.text(
            negative_input_ids,
            negative_attention_mask,
        )[0]
        self.scheduler.set_timesteps(num_inference_steps=self.num_infer_timesteps)
        pipeline = StableDiffusionPipeline(
            vae=self.vae,
            text_encoder=self.text,
            unet=self.unet,
            scheduler=self.scheduler,
            tokenizer=None,
            safety_checker=None,
            feature_extractor=None,
        )
        pipeline.set_progress_bar_config(disable=True)

        images = pipeline(
            prompt_embeds=prompt_embeds,
            negative_prompt_embeds=negative_prompt_embeds,
            generator=torch.Generator(device=pipeline.device).manual_seed(self.seed),
            guidance_scale=guidance_scale,
            output_type="np.array",
        ).images

        return GenericOutputs(images=torch.from_numpy(images))


class StableForImageInpainting(GenericModel):
    pass


class StableForImageResolution(GenericModel):
    pass



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.models.chatglm.modeling import (
    ChatGLMForClassification,
    ChatGLMForPretrain,
    ChatGLMForGeneration,
)
from unitorch.models.chatglm.processing import ChatGLMProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import json
import torch
import torch.nn as nn
import torch.nn.functional as F
import transformers
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers import (
    ProphetNetConfig,
    ProphetNetModel,
    ProphetNetForConditionalGeneration,
)
from unitorch.utils.decorators import replace


@replace(transformers.models.prophetnet.modeling_prophetnet.ProphetNetAttention)
class ProphetNetAttentionV2(
    transformers.models.prophetnet.modeling_prophetnet.ProphetNetAttention
):
    def __init__(
        self,
        config: ProphetNetConfig,
        num_attn_heads: int,
    ):
        super().__init__(config=config, num_attn_heads=num_attn_heads)

    def _shape(self, tensor: torch.Tensor, seq_len: int, bsz: int):
        return (
            tensor.view(bsz, seq_len, self.num_attn_heads, self.head_dim)
            .transpose(1, 2)
            .contiguous()
        )

    def forward(
        self,
        hidden_states,
        key_value_states: Optional[torch.Tensor] = None,
        attention_mask: Optional[torch.Tensor] = None,
        layer_head_mask: Optional[torch.Tensor] = None,
        past_key_value: Optional[Tuple[torch.Tensor]] = None,
        output_attentions: Optional[bool] = False,
    ) -> Tuple[torch.Tensor, Optional[torch.Tensor]]:
        batch_size, tgt_len, hidden_size = hidden_states.size()
        kv_batch_size = batch_size

        # if key_value_states are provided this layer is used as a cross-attention layer
        # for the decoder
        is_cross_attention = key_value_states is not None
        assert list(hidden_states.size()) == [
            batch_size,
            tgt_len,
            hidden_size,
        ], f"Size of hidden states should be {batch_size, tgt_len, hidden_size}, but is {hidden_states.size()}"

        # previous time steps are cached - no need to recompute key and value if they are static
        query_states = self.query_proj(hidden_states) / (self.head_dim**0.5)

        if is_cross_attention and past_key_value is not None:
            # reuse k,v, cross_attentions
            key_states = past_key_value[0]
            value_states = past_key_value[1]
            kv_batch_size = key_states.size(0)
        elif is_cross_attention:
            # cross_attentions
            kv_batch_size = key_value_states.size(0)
            key_states = self._shape(self.key_proj(key_value_states), -1, kv_batch_size)
            value_states = self._shape(
                self.value_proj(key_value_states), -1, kv_batch_size
            )
        else:
            # self_attention
            key_states = self._shape(self.key_proj(hidden_states), -1, batch_size)
            value_states = self._shape(self.value_proj(hidden_states), -1, batch_size)

        if is_cross_attention:
            # if cross_attention save Tuple(torch.Tensor, torch.Tensor) of all cross attention key/value_states.
            # Further calls to cross_attention layer can then reuse all cross-attention
            # key/value_states (first "if" case)
            # if encoder bi-directional self-attention `past_key_value` is always `None`
            past_key_value = (key_states, value_states)

        # project states into the correct shape
        proj_shape = (batch_size, self.num_attn_heads, -1, self.head_dim)
        query_states = self._shape(query_states, tgt_len, batch_size).view(*proj_shape)
        kv_proj_shape = (kv_batch_size, self.num_attn_heads, -1, self.head_dim)
        key_states = key_states.view(*kv_proj_shape)
        value_states = value_states.view(*kv_proj_shape)

        src_len = key_states.size(2)
        if is_cross_attention and kv_batch_size != batch_size:
            attn_weights = torch.einsum(
                "bxhtd,bhsd->bxhts",
                query_states.view(kv_batch_size, -1, *query_states.size()[1:]),
                key_states.view(kv_batch_size, *key_states.size()[1:]),
            )
            attn_weights = attn_weights.reshape(-1, *attn_weights.size()[-3:])
        else:
            attn_weights = torch.einsum(
                "bsij,bsjk->bsik", query_states, key_states.transpose(2, 3)
            )

        expected_shape = (batch_size, self.num_attn_heads, tgt_len, src_len)
        if attn_weights.size() != expected_shape:
            raise ValueError(
                f"Attention weights should have size {expected_shape}, but is {attn_weights.size()}"
            )

        # This is part of a workaround to get around fork/join parallelism not supporting Optional types.

        if attention_mask is not None and attention_mask.dim() == 0:
            attention_mask = None
        expected_shape = (batch_size, self.num_attn_heads, 1, src_len)
        if attention_mask is not None and attention_mask.size() != expected_shape:
            raise ValueError(
                f"Attention mask should have size {expected_shape}, but is {attention_mask.size()}"
            )
        if attention_mask is not None:  # don't attend to padding symbols
            attn_weights = attn_weights + attention_mask
        if output_attentions:
            attn_weights_reshaped = attn_weights
        else:
            attn_weights_reshaped = None

        attn_weights = F.softmax(attn_weights, dim=-1)
        if layer_head_mask is not None:
            assert layer_head_mask.size() == (
                self.num_attn_heads,
            ), f"Head mask for a single layer should be of size {(self.num_attn_heads,)}, but is {layer_head_mask.size()}"
            attn_weights = layer_head_mask.view(1, -1, 1, 1) * attn_weights.view(
                batch_size, self.num_attn_heads, tgt_len, src_len
            )
            attn_weights = attn_weights.view(
                batch_size * self.num_attn_heads, tgt_len, src_len
            )

            # apply head_mask also on attn_weights_reshaped which is used for n-gram attention inside the model
            attn_weights_reshaped = (
                layer_head_mask.view(1, -1, 1, 1) * attn_weights_reshaped
            )

        attn_probs = F.dropout(
            attn_weights,
            p=self.attention_dropout,
            training=self.training,
        )

        if is_cross_attention and kv_batch_size != batch_size:
            attn_probs = attn_probs.to(value_states.dtype)
            attn_output = torch.einsum(
                "bxhts,bhsd->bxhtd",
                attn_probs.view(kv_batch_size, -1, *attn_probs.size()[1:]),
                value_states.view(kv_batch_size, *value_states.size()[1:]),
            )
            attn_output = attn_output.reshape(-1, *attn_output.size()[-3:])
        else:
            attn_output = torch.einsum("bsij,bsjk->bsik", attn_probs, value_states)

        expected_shape = (batch_size, self.num_attn_heads, tgt_len, self.head_dim)
        if attn_output.size() != expected_shape:
            raise ValueError(
                f"`attn_output` should have shape {expected_shape}, but is of shape {attn_output.size()}"
            )

        attn_output = attn_output.transpose(1, 2).reshape(
            batch_size, tgt_len, hidden_size
        )
        attn_output = self.out_proj(attn_output)

        attn_output = nn.functional.dropout(
            attn_output, p=self.dropout, training=self.training
        )
        return attn_output, attn_weights_reshaped, past_key_value



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import json
import random
from functools import partial
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union

import numpy as np
import torch

from unitorch.models.chatglm.tokenization_chatglm import ChatGLMTokenizer
from unitorch.utils import pop_value, truncate_sequence_pair
from unitorch.models import (
    HfTextClassificationProcessor,
    HfTextGenerationProcessor,
    GenericOutputs,
)


class ChatGLMProcessor(HfTextClassificationProcessor, HfTextGenerationProcessor):
    """
    Processor for handling text classification and text generation tasks in a chat-based conversational model.

    Args:
        vocab_file (str): Path to the vocabulary file.
        tokenizer_file (str): Path to the tokenizer file.
        max_seq_length (int, optional): Maximum sequence length for text classification. Defaults to 128.
        max_gen_seq_length (int, optional): Maximum sequence length for text generation. Defaults to 48.
    """

    def __init__(
        self,
        vocab_file: str,
        tokenizer_file: str,
        max_seq_length: Optional[int] = 128,
        max_gen_seq_length: Optional[int] = 48,
    ):
        if not os.path.exists(tokenizer_file):
            config_dict = {}
        else:
            config_dict = json.loads(open(tokenizer_file, "r").read())
        tokenizer = ChatGLMTokenizer(vocab_file=vocab_file, **config_dict)
        HfTextClassificationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
        )
        HfTextGenerationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )
        self.gmask_token = self.tokenizer.gmask_token

    def classification(
        self,
        text: str,
        text_pair: Optional[str] = None,
        max_seq_length: Optional[int] = None,
    ):
        """
        Preprocesses text for classification task.

        Args:
            text (str): The input text to classify.
            text_pair (str, optional): The second input text for sequence pair classification. Defaults to None.
            max_seq_length (int, optional): Maximum sequence length. Overrides the default value set during initialization.

        Returns:
            GenericOutputs: Preprocessed input for classification task.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )

        tokens = self.tokenizer.tokenize(str(text))
        if text_pair is None:
            tokens = tokens[: max_seq_length - 2]
            tokens = tokens + [self.gmask_token] + [self.bos_token]
            input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        else:
            tokens_pair = self.tokenizer.tokenize(str(text_pair))
            truncate_sequence_pair(tokens, tokens_pair, max_seq_length - 2)
            tokens = tokens + tokens_pair + [self.gmask_token, self.bos_token]
            input_ids = self.tokenizer.convert_tokens_to_ids(tokens)

        padding = [0] * (max_seq_length - len(input_ids))
        input_ids = len(padding) * [self.pad_token_id] + input_ids

        attention_mask = torch.ones(max_seq_length, max_seq_length, dtype=torch.long)
        attention_mask[len(padding) :, len(padding) : max_seq_length - 1] = 0
        attention_mask[max_seq_length - 1, max_seq_length - 1] = 0

        position_ids1 = [0] * len(padding) + list(range(0, len(tokens)))
        position_ids2 = [0] * (max_seq_length - 1) + [1]

        assert len(input_ids) == max_seq_length
        assert len(attention_mask) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.bool).unsqueeze(0),
            position_ids=torch.tensor(
                [position_ids1, position_ids2],
                dtype=torch.long,
            ),
        )

    def prompt(
        self,
        text: str,
        max_seq_length: Optional[int] = None,
    ):
        """
        Preprocesses input text for prompt.

        Args:
            text (str): The input text for the prompt.
            max_seq_length (int, optional): Maximum sequence length for classification. Overrides the default value set during initialization.

        Returns:
            GenericOutputs: Preprocessed input for prompt.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        tokens = self.tokenizer.tokenize(str(text))[: max_seq_length - 2] + [
            self.gmask_token,
            self.bos_token,
        ]
        padding = [self.pad_token] * (max_seq_length - len(tokens))
        tokens = padding + tokens
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)

        assert len(input_ids) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
        )

    def generation_inputs(
        self,
        text: str,
        max_seq_length: Optional[int] = None,
    ):
        """
        Preprocesses input text for generation prompt.

        Args:
            text (str): The input text for the generation prompt.
            max_seq_length (int, optional): Maximum sequence length for generation. Overrides the default value set during initialization.

        Returns:
            GenericOutputs: Preprocessed input for generation prompt.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        tokens = self.tokenizer.tokenize(str(text))[: max_seq_length - 2] + [
            self.gmask_token,
            self.bos_token,
        ]
        padding = [self.pad_token] * (max_seq_length - len(tokens))
        tokens = padding + tokens
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)

        assert len(input_ids) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
        )

    def generation_labels(
        self,
        text: str,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Preprocesses input text for generation labels.

        Args:
            text (str): The input text for generation labels.
            max_gen_seq_length (int, optional): Maximum sequence length for generation labels. Overrides the default value set during initialization.

        Returns:
            GenericOutputs: Preprocessed input for generation labels.
        """
        max_gen_seq_length = pop_value(
            max_gen_seq_length,
            self.max_gen_seq_length,
        )
        tokens = self.tokenizer.tokenize(str(text))[: max_gen_seq_length - 1] + [
            self.eos_token
        ]
        padding = [self.pad_token] * (max_gen_seq_length - len(tokens))
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        attention_mask = [1] * len(input_ids)

        padding = [0] * (max_gen_seq_length - len(input_ids))
        input_ids += [self.pad_token_id] * len(padding)
        attention_mask += padding

        assert len(input_ids) == max_gen_seq_length
        assert len(attention_mask) == max_gen_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
        )

    def generation(
        self,
        text: str,
        text_pair: str,
        max_seq_length: Optional[int] = None,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Preprocesses text for generation task.

        Args:
            text (str): The input text for the generation prompt.
            text_pair (str): The input text for conditioning or continuation.
            max_seq_length (int, optional): Maximum sequence length for classification. Overrides the default value set during initialization.
            max_gen_seq_length (int, optional): Maximum sequence length for generation. Overrides the default value set during initialization.

        Returns:
            GenericOutputs: Preprocessed input for generation task.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        max_gen_seq_length = pop_value(
            max_gen_seq_length,
            self.max_gen_seq_length,
        )

        tokens = self.tokenizer.tokenize(str(text))[1 - max_seq_length :] + [
            self.gmask_token
        ]
        tokens_pair = (
            [self.bos_token]
            + self.tokenizer.tokenize(str(text_pair))[: max_gen_seq_length - 2]
            + [self.eos_token]
        )
        padding_a = [self.pad_token] * (max_seq_length - len(tokens))
        padding_b = [self.pad_token] * (max_gen_seq_length - len(tokens_pair))
        _tokens = padding_a + tokens + tokens_pair + padding_b
        input_ids = self.tokenizer.convert_tokens_to_ids(_tokens)

        tokens_label = tokens_pair[1:max_gen_seq_length] + [self.pad_token] * (
            max_gen_seq_length - len(tokens_pair) + 1
        )
        input_ids_label = self.tokenizer.convert_tokens_to_ids(tokens_label)
        input_ids_label = [0] * max_seq_length + input_ids_label
        attention_mask_label = [1] * len(tokens_pair[1:max_gen_seq_length]) + [0] * (
            max_gen_seq_length - len(tokens_pair) + 1
        )
        attention_mask_label = [0] * max_seq_length + attention_mask_label

        attention_mask = torch.ones(
            max_seq_length + max_gen_seq_length,
            max_seq_length + max_gen_seq_length,
            dtype=torch.long,
        )
        attention_mask[
            len(padding_a) : max_seq_length + len(tokens_pair),
            len(padding_a) : len(padding_a) + len(tokens),
        ] = 0
        attention_mask[
            max_seq_length : max_seq_length + len(tokens_pair),
            max_seq_length : max_seq_length + len(tokens_pair),
        ] = 1 - torch.tril(torch.ones(len(tokens_pair), len(tokens_pair)))
        position_ids_a = (
            [0] * len(padding_a)
            + list(range(0, len(tokens)))
            + [max_seq_length] * max_gen_seq_length
        )
        position_ids_b = (
            [0] * max_seq_length
            + list(range(1, len(tokens_pair)))
            + [len(tokens_pair)] * (max_gen_seq_length - len(tokens_pair) + 1)
        )

        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.bool).unsqueeze(0),
            position_ids=torch.tensor(
                [position_ids_a, position_ids_b],
                dtype=torch.long,
            ),
            input_ids_label=torch.tensor(input_ids_label, dtype=torch.long),
            attention_mask_label=torch.tensor(attention_mask_label, dtype=torch.long),
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import unitorch.cli.models.peft.modeling_bloom
import unitorch.cli.models.peft.modeling_llama
from unitorch.cli.models.peft.modeling_bloom import (
    BloomAdaLoraForClassification,
    BloomAdaLoraForGeneration,
    BloomLoraForClassification,
    BloomLoraForGeneration,
)
from unitorch.cli.models.peft.modeling_llama import (
    LlamaAdaLoraForClassification,
    LlamaAdaLoraForGeneration,
    LlamaLoraForClassification,
    LlamaLoraForGeneration,
)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
import torch.nn as nn
from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
from transformers.models.deberta.modeling_deberta import (
    DebertaConfig,
    DebertaModel,
    DebertaOnlyMLMHead,
    ContextPooler,
    StableDropout,
)
from unitorch.models import GenericModel


class DebertaForClassification(GenericModel):
    def __init__(
        self,
        config_path: str,
        num_classes: Optional[int] = 1,
        gradient_checkpointing: Optional[bool] = False,
    ):
        """
        Initializes the DebertaForClassification model.

        Args:
            config_path (str): The path to the Deberta model configuration file.
            num_classes (int, optional): The number of classes for classification. Defaults to 1.
            gradient_checkpointing (bool, optional): Whether to use gradient checkpointing for memory optimization. Defaults to False.
        """
        super().__init__()
        self.config = DebertaConfig.from_json_file(config_path)
        self.config.gradient_checkpointing = gradient_checkpointing
        self.deberta = DebertaModel(self.config)
        self.pooler = ContextPooler(self.config)
        self.dropout = StableDropout(self.config.hidden_dropout_prob)
        self.classifier = nn.Linear(self.config.hidden_size, num_classes)
        self.init_weights()

    def forward(
        self,
        input_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        token_type_ids: Optional[torch.Tensor] = None,
        position_ids: Optional[torch.Tensor] = None,
    ):
        """
        Performs forward pass of the DebertaForClassification model.

        Args:
            input_ids (torch.Tensor): The input tensor containing token ids.
            attention_mask (torch.Tensor, optional): The attention mask tensor. Defaults to None.
            token_type_ids (torch.Tensor, optional): The token type ids tensor. Defaults to None.
            position_ids (torch.Tensor, optional): The position ids tensor. Defaults to None.

        Returns:
            (torch.Tensor):The output logits tensor.
        """
        outputs = self.deberta(
            input_ids,
            attention_mask=attention_mask,
            token_type_ids=token_type_ids,
            position_ids=position_ids,
        )
        pooled_output = self.pooler(outputs[0])

        pooled_output = self.dropout(pooled_output)
        logits = self.classifier(pooled_output)
        return logits


class DebertaForMaskLM(GenericModel):
    def __init__(
        self,
        config_path: str,
        gradient_checkpointing: Optional[bool] = False,
    ):
        """
        Initializes the DebertaForMaskLM model.

        Args:
            config_path (str): The path to the Deberta model configuration file.
            gradient_checkpointing (bool, optional): Whether to use gradient checkpointing for memory optimization. Defaults to False.
        """
        super().__init__()
        self.config = DebertaConfig.from_json_file(config_path)
        self.config.gradient_checkpointing = gradient_checkpointing
        self.deberta = DebertaModel(self.config)
        self.cls = DebertaOnlyMLMHead(self.config)
        self.cls.predictions.decoder.weight = (
            self.deberta.embeddings.word_embeddings.weight
        )
        self.init_weights()

    def forward(
        self,
        input_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        token_type_ids: Optional[torch.Tensor] = None,
        position_ids: Optional[torch.Tensor] = None,
    ):
        """
        Performs forward pass of the DebertaForMaskLM model.

        Args:
            input_ids (torch.Tensor): The input tensor containing token ids.
            attention_mask (torch.Tensor, optional): The attention mask tensor. Defaults to None.
            token_type_ids (torch.Tensor, optional): The token type ids tensor. Defaults to None.
            position_ids (torch.Tensor, optional): The position ids tensor. Defaults to None.

        Returns:
            (torch.Tensor):The output logits tensor.
        """
        outputs = self.deberta(
            input_ids,
            attention_mask=attention_mask,
            token_type_ids=token_type_ids,
            position_ids=position_ids,
        )
        sequence_output = outputs[0]
        logits = self.cls(sequence_output)
        return logits



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from torch import device, Tensor
import json
import torch
import torch.nn as nn
import torch.nn.functional as F
import transformers
from transformers import (
    XLMProphetNetConfig,
    XLMProphetNetModel,
    XLMProphetNetForConditionalGeneration,
)

from unitorch.models import GenericModel, GenericOutputs
from unitorch.utils.decorators import replace


@replace(
    transformers.models.xlm_prophetnet.modeling_xlm_prophetnet.XLMProphetNetAttention
)
class ProphetNetAttentionV2(
    transformers.models.xlm_prophetnet.modeling_xlm_prophetnet.XLMProphetNetAttention
):
    def __init__(
        self,
        config: XLMProphetNetConfig,
        num_attn_heads: int,
    ):
        super().__init__(config=config, num_attn_heads=num_attn_heads)

    def _shape(self, tensor: torch.Tensor, seq_len: int, bsz: int):
        return (
            tensor.view(bsz, seq_len, self.num_attn_heads, self.head_dim)
            .transpose(1, 2)
            .contiguous()
        )

    def forward(
        self,
        hidden_states,
        key_value_states: Optional[torch.Tensor] = None,
        attention_mask: Optional[torch.Tensor] = None,
        layer_head_mask: Optional[torch.Tensor] = None,
        past_key_value: Optional[Tuple[torch.Tensor]] = None,
        output_attentions: Optional[bool] = False,
    ) -> Tuple[torch.Tensor, Optional[torch.Tensor]]:
        batch_size, tgt_len, hidden_size = hidden_states.size()
        kv_batch_size = batch_size

        # if key_value_states are provided this layer is used as a cross-attention layer
        # for the decoder
        is_cross_attention = key_value_states is not None
        assert list(hidden_states.size()) == [
            batch_size,
            tgt_len,
            hidden_size,
        ], f"Size of hidden states should be {batch_size, tgt_len, hidden_size}, but is {hidden_states.size()}"

        # previous time steps are cached - no need to recompute key and value if they are static
        query_states = self.query_proj(hidden_states) / (self.head_dim**0.5)

        if is_cross_attention and past_key_value is not None:
            # reuse k,v, cross_attentions
            key_states = past_key_value[0]
            value_states = past_key_value[1]
            kv_batch_size = key_states.size(0)
        elif is_cross_attention:
            # cross_attentions
            kv_batch_size = key_value_states.size(0)
            key_states = self._shape(self.key_proj(key_value_states), -1, kv_batch_size)
            value_states = self._shape(
                self.value_proj(key_value_states), -1, kv_batch_size
            )
        else:
            # self_attention
            key_states = self._shape(self.key_proj(hidden_states), -1, batch_size)
            value_states = self._shape(self.value_proj(hidden_states), -1, batch_size)

        if is_cross_attention:
            # if cross_attention save Tuple(torch.Tensor, torch.Tensor) of all cross attention key/value_states.
            # Further calls to cross_attention layer can then reuse all cross-attention
            # key/value_states (first "if" case)
            # if encoder bi-directional self-attention `past_key_value` is always `None`
            past_key_value = (key_states, value_states)

        # project states into the correct shape
        proj_shape = (batch_size, self.num_attn_heads, -1, self.head_dim)
        query_states = self._shape(query_states, tgt_len, batch_size).view(*proj_shape)
        kv_proj_shape = (kv_batch_size, self.num_attn_heads, -1, self.head_dim)
        key_states = key_states.view(*kv_proj_shape)
        value_states = value_states.view(*kv_proj_shape)

        src_len = key_states.size(2)
        if is_cross_attention and kv_batch_size != batch_size:
            attn_weights = torch.einsum(
                "bxhtd,bhsd->bxhts",
                query_states.view(kv_batch_size, -1, *query_states.size()[1:]),
                key_states.view(kv_batch_size, *key_states.size()[1:]),
            )
            attn_weights = attn_weights.reshape(-1, *attn_weights.size()[-3:])
        else:
            attn_weights = torch.einsum(
                "bsij,bsjk->bsik", query_states, key_states.transpose(2, 3)
            )

        expected_shape = (batch_size, self.num_attn_heads, tgt_len, src_len)
        if attn_weights.size() != expected_shape:
            raise ValueError(
                f"Attention weights should have size {expected_shape}, but is {attn_weights.size()}"
            )

        # This is part of a workaround to get around fork/join parallelism not supporting Optional types.

        if attention_mask is not None and attention_mask.dim() == 0:
            attention_mask = None
        expected_shape = (batch_size, self.num_attn_heads, 1, src_len)
        if attention_mask is not None and attention_mask.size() != expected_shape:
            raise ValueError(
                f"Attention mask should have size {expected_shape}, but is {attention_mask.size()}"
            )
        if attention_mask is not None:  # don't attend to padding symbols
            attn_weights = attn_weights + attention_mask
        if output_attentions:
            attn_weights_reshaped = attn_weights
        else:
            attn_weights_reshaped = None

        attn_weights = F.softmax(attn_weights, dim=-1)
        if layer_head_mask is not None:
            assert layer_head_mask.size() == (
                self.num_attn_heads,
            ), f"Head mask for a single layer should be of size {(self.num_attn_heads,)}, but is {layer_head_mask.size()}"
            attn_weights = layer_head_mask.view(1, -1, 1, 1) * attn_weights.view(
                batch_size, self.num_attn_heads, tgt_len, src_len
            )
            attn_weights = attn_weights.view(
                batch_size * self.num_attn_heads, tgt_len, src_len
            )

            # apply head_mask also on attn_weights_reshaped which is used for n-gram attention inside the model
            attn_weights_reshaped = (
                layer_head_mask.view(1, -1, 1, 1) * attn_weights_reshaped
            )

        attn_probs = F.dropout(
            attn_weights,
            p=self.attention_dropout,
            training=self.training,
        )

        if is_cross_attention and kv_batch_size != batch_size:
            attn_probs = attn_probs.to(value_states.dtype)
            attn_output = torch.einsum(
                "bxhts,bhsd->bxhtd",
                attn_probs.view(kv_batch_size, -1, *attn_probs.size()[1:]),
                value_states.view(kv_batch_size, *value_states.size()[1:]),
            )
            attn_output = attn_output.reshape(-1, *attn_output.size()[-3:])
        else:
            attn_output = torch.einsum("bsij,bsjk->bsik", attn_probs, value_states)

        expected_shape = (batch_size, self.num_attn_heads, tgt_len, self.head_dim)
        if attn_output.size() != expected_shape:
            raise ValueError(
                f"`attn_output` should have shape {expected_shape}, but is of shape {attn_output.size()}"
            )

        attn_output = attn_output.transpose(1, 2).reshape(
            batch_size, tgt_len, hidden_size
        )
        attn_output = self.out_proj(attn_output)

        attn_output = nn.functional.dropout(
            attn_output, p=self.dropout, training=self.training
        )
        return attn_output, attn_weights_reshaped, past_key_value


class XProphetNetForGeneration(GenericModel):
    prefix_keys_in_state_dict = {"^(?!model\.).*": "model."}

    def __init__(
        self,
        config_path: str,
        freeze_input_embedding: Optional[bool] = True,
        gradient_checkpointing: Optional[bool] = False,
    ):
        super().__init__()

        self.config = XLMProphetNetConfig.from_json_file(config_path)
        self.config.gradient_checkpointing = gradient_checkpointing
        self.model = XLMProphetNetForConditionalGeneration(self.config)

        if freeze_input_embedding:
            for param in self.model.get_input_embeddings().parameters():
                param.requires_grad = False

        self.init_weights()

    def forward(
        self,
        input_ids: torch.Tensor,
        attention_mask: torch.Tensor,
        decoder_input_ids: torch.Tensor,
        decoder_attention_mask: torch.Tensor,
    ):
        """
        Performs forward pass of the XPegasusForGeneration model.

        Args:
            input_ids (torch.Tensor): Tensor of input token IDs.
            attention_mask (torch.Tensor): Tensor of attention mask.
            decoder_input_ids (torch.Tensor): Tensor of decoder input token IDs.
            decoder_attention_mask (torch.Tensor): Tensor of decoder attention mask.

        Returns:
            (torch.Tensor):The model's logits.
        """
        outputs = self.model(
            input_ids=input_ids,
            attention_mask=attention_mask,
            decoder_input_ids=decoder_input_ids,
            decoder_attention_mask=decoder_attention_mask,
            return_dict=True,
        )
        logits = outputs.logits.unsqueeze(1)
        if outputs.logits_ngram is not None:
            logits_ngram = outputs.logits_ngram
            logits = torch.cat([logits, logits_ngram], dim=1)
        return logits

    @torch.no_grad()
    def generate(
        self,
        input_ids: torch.Tensor,
        num_beams: Optional[int] = 5,
        decoder_start_token_id: Optional[int] = 2,
        decoder_end_token_id: Optional[int] = 2,
        num_return_sequences: Optional[int] = 1,
        min_gen_seq_length: Optional[int] = 0,
        max_gen_seq_length: Optional[int] = 48,
        repetition_penalty: Optional[float] = 1.0,
        no_repeat_ngram_size: Optional[int] = 0,
        early_stopping: Optional[bool] = True,
        length_penalty: Optional[float] = 1.0,
        num_beam_groups: Optional[int] = 1,
        diversity_penalty: Optional[float] = 0.0,
        do_sample: Optional[bool] = False,
        temperature: Optional[float] = 1.0,
        top_k: Optional[int] = 50,
        top_p: Optional[float] = 1.0,
    ):
        """
        Generates sequences using the XPegasusForGeneration model.

        Args:
            input_ids: The input token IDs.
            num_beams (int, optional): The number of beams for beam search. Defaults to 5.
            decoder_start_token_id (int, optional): The decoder's start token ID. Defaults to 2.
            decoder_end_token_id (int, optional): The decoder's end token ID. Defaults to 2.
            num_return_sequences (int, optional): The number of generated sequences to return. Defaults to 1.
            min_gen_seq_length (int, optional): The minimum length of the generated sequences. Defaults to 0.
            max_gen_seq_length (int, optional): The maximum length of the generated sequences. Defaults to 48.
            repetition_penalty (float, optional): The repetition penalty. Defaults to 1.0.
            no_repeat_ngram_size (int, optional): The size of n-grams to avoid repeating. Defaults to 0.
            early_stopping (bool, optional): Whether to stop generation early. Defaults to True.
            length_penalty (float, optional): The length penalty. Defaults to 1.0.
            num_beam_groups (int, optional): The number of beam groups for diverse beam search. Defaults to 1.
            diversity_penalty (float, optional): The diversity penalty. Defaults to 0.0.
            do_sample (bool, optional): Whether to use sampling for generation. Defaults to False.
            temperature (float, optional): The temperature for sampling. Defaults to 1.0.
            top_k (int, optional): The value for top-k sampling. Defaults to 50.
            top_p (float, optional): The value for top-p (nucleus) sampling. Defaults to 1.0.

        Returns:
            GenericOutputs: The generated sequences and their scores.
        """
        outputs = self.model.generate(
            input_ids,
            max_length=max_gen_seq_length,
            min_length=min_gen_seq_length,
            num_beams=num_beams,
            do_sample=do_sample,
            decoder_start_token_id=decoder_start_token_id,
            no_repeat_ngram_size=no_repeat_ngram_size,
            early_stopping=early_stopping,
            length_penalty=length_penalty,
            repetition_penalty=repetition_penalty,
            num_return_sequences=num_return_sequences,
            bos_token_id=decoder_start_token_id,
            eos_token_id=decoder_end_token_id,
            num_beam_groups=num_beam_groups,
            diversity_penalty=diversity_penalty,
            temperature=temperature,
            top_k=top_k,
            top_p=top_p,
            return_dict_in_generate=True,
            output_scores=True,
        )

        sequences = outputs.sequences.reshape(
            -1, num_return_sequences, outputs.sequences.size(-1)
        )
        outputs.sequences = torch.zeros(
            sequences.size(0), num_return_sequences, max_gen_seq_length
        ).to(device=sequences.device)
        outputs.sequences[:, :, : sequences.size(-1)].copy_(sequences)

        if num_return_sequences == 1:
            outputs.sequences = outputs.sequences.reshape(-1, max_gen_seq_length)

        return GenericOutputs(
            sequences=outputs.sequences,
            sequences_scores=outputs.sequences_scores,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

__hf_hub_stable_dict__ = lambda name: {
    "unet": {
        "config": f"https://huggingface.co/{name}/resolve/main/unet/config.json",
        "weight": f"https://huggingface.co/{name}/resolve/main/unet/diffusion_pytorch_model.bin",
    },
    "text": {
        "config": f"https://huggingface.co/{name}/resolve/main/text_encoder/config.json",
        "vocab": f"https://huggingface.co/{name}/resolve/main/tokenizer/vocab.json",
        "merge": f"https://huggingface.co/{name}/resolve/main/tokenizer/merges.txt",
        "weight": f"https://huggingface.co/{name}/resolve/main/text_encoder/pytorch_model.bin",
    },
    "vae": {
        "config": f"https://huggingface.co/{name}/resolve/main/vae/config.json",
        "weight": f"https://huggingface.co/{name}/resolve/main/vae/diffusion_pytorch_model.bin",
    },
    "scheduler": f"https://huggingface.co/{name}/resolve/main/scheduler/scheduler_config.json",
}

__hf_hub_controlnet_dict__ = lambda name: {
    **__hf_hub_stable_dict__("runwayml/stable-diffusion-v1-5"),
    **{
        "controlnet": {
            "config": f"https://huggingface.co/{name}/resolve/main/config.json",
            "weight": f"https://huggingface.co/{name}/resolve/main/diffusion_pytorch_model.bin",
        }
    },
}

pretrained_diffusers_infos = {
    "ddpm-ema-pokemon-64": {
        "config": "https://huggingface.co/anton-l/ddpm-ema-pokemon-64/resolve/main/unet/config.json",
        "weight": "https://huggingface.co/anton-l/ddpm-ema-pokemon-64/resolve/main/unet/diffusion_pytorch_model.bin",
    },
    "stable-v1.5": __hf_hub_stable_dict__("runwayml/stable-diffusion-v1-5"),
    "stable-v2": __hf_hub_stable_dict__("stabilityai/stable-diffusion-2"),
    "stable-v2.1": __hf_hub_stable_dict__("stabilityai/stable-diffusion-2-1"),
    "stable-x4-upscaler": __hf_hub_stable_dict__(
        "stabilityai/stable-diffusion-x4-upscaler"
    ),
    "controlnet-canny": __hf_hub_controlnet_dict__("lllyasviel/sd-controlnet-canny"),
    "controlnet-openpose": __hf_hub_controlnet_dict__(
        "lllyasviel/sd-controlnet-openpose"
    ),
}
from unitorch import is_diffusers_available

if is_diffusers_available():
    import unitorch.cli.models.diffusers.modeling_controlnet
    import unitorch.cli.models.diffusers.modeling_stable
    import unitorch.cli.models.diffusers.processing_controlnet
    import unitorch.cli.models.diffusers.processing_stable



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import torch
import logging
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.optim import SGD, Adam, AdamW
from unitorch.models import CheckpointMixin
from unitorch.cli import add_default_section_for_init, register_optim


@register_optim("core/optim/sgd")
class SGDOptimizer(SGD, CheckpointMixin):
    def __init__(
        self,
        params,
        learning_rate: Optional[float] = 0.00001,
    ):
        super().__init__(
            params=params,
            lr=learning_rate,
        )

    @classmethod
    @add_default_section_for_init("core/optim/sgd")
    def from_core_configure(cls, config, **kwargs):
        pass


@register_optim("core/optim/adam")
class AdamOptimizer(Adam, CheckpointMixin):
    def __init__(
        self,
        params,
        learning_rate: Optional[float] = 0.00001,
    ):
        super().__init__(
            params=params,
            lr=learning_rate,
        )

    @classmethod
    @add_default_section_for_init("core/optim/adam")
    def from_core_configure(cls, config, **kwargs):
        pass


@register_optim("core/optim/adamw")
class AdamWOptimizer(AdamW, CheckpointMixin):
    def __init__(
        self,
        params,
        learning_rate: Optional[float] = 0.00001,
    ):
        super().__init__(
            params=params,
            lr=learning_rate,
        )

    @classmethod
    @add_default_section_for_init("core/optim/adamw")
    def from_core_configure(cls, config, **kwargs):
        pass


# more optims
import unitorch.cli.optim.lion



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from torch.cuda.amp import autocast

from unitorch.models.diffusers import (
    ControlNetForImageGeneration as _ControlNetForImageGeneration,
)
from unitorch.utils import pop_value, nested_dict_value
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_model,
)
from unitorch.cli.models import DiffusionOutputs, LossOutputs
from unitorch.cli.models import diffusion_model_decorator
from unitorch.cli.models.diffusers import pretrained_diffusers_infos


@register_model("core/model/diffusion/controlnet", diffusion_model_decorator)
class ControlNetForImageGeneration(_ControlNetForImageGeneration):
    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        vae_config_path: str,
        controlnet_config_path: str,
        scheduler_config_path: str,
        image_size: Optional[int] = 224,
        in_channels: Optional[int] = 4,
        out_channels: Optional[int] = 4,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        freeze_vae_encoder: Optional[bool] = True,
        freeze_text_encoder: Optional[bool] = True,
        freeze_unet_encoder: Optional[bool] = True,
        seed: Optional[int] = 1123,
    ):
        super().__init__(
            config_path=config_path,
            text_config_path=text_config_path,
            vae_config_path=vae_config_path,
            controlnet_config_path=controlnet_config_path,
            scheduler_config_path=scheduler_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            freeze_vae_encoder=freeze_vae_encoder,
            freeze_text_encoder=freeze_text_encoder,
            freeze_unet_encoder=freeze_unet_encoder,
            seed=seed,
        )
        for n, p in self.named_parameters():
            if "blocks" in n:
                p.requires_grad = False

    @classmethod
    @add_default_section_for_init("core/model/diffusion/controlnet")
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section("core/model/diffusion/controlnet")
        pretrained_name = config.getoption("pretrained_name", "controlnet-canny")
        pretrain_infos = nested_dict_value(pretrained_diffusers_infos, pretrained_name)

        config_path = config.getoption("config_path", None)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrain_infos, "unet", "config"),
        )
        config_path = cached_path(config_path)

        text_config_path = config.getoption("text_config_path", None)
        text_config_path = pop_value(
            text_config_path,
            nested_dict_value(pretrain_infos, "text", "config"),
        )
        text_config_path = cached_path(text_config_path)

        vae_config_path = config.getoption("vae_config_path", None)
        vae_config_path = pop_value(
            vae_config_path,
            nested_dict_value(pretrain_infos, "vae", "config"),
        )
        vae_config_path = cached_path(vae_config_path)

        controlnet_config_path = config.getoption("controlnet_config_path", None)
        controlnet_config_path = pop_value(
            controlnet_config_path,
            nested_dict_value(pretrain_infos, "controlnet", "config"),
        )
        controlnet_config_path = cached_path(controlnet_config_path)

        scheduler_config_path = config.getoption("scheduler_config_path", None)
        scheduler_config_path = pop_value(
            scheduler_config_path,
            nested_dict_value(pretrain_infos, "scheduler"),
        )
        scheduler_config_path = cached_path(scheduler_config_path)

        image_size = config.getoption("image_size", 224)
        in_channels = config.getoption("in_channels", 4)
        out_channels = config.getoption("out_channels", 4)
        num_train_timesteps = config.getoption("num_train_timesteps", 1000)
        num_infer_timesteps = config.getoption("num_infer_timesteps", 50)
        freeze_vae_encoder = config.getoption("freeze_vae_encoder", True)
        freeze_text_encoder = config.getoption("freeze_text_encoder", True)
        freeze_unet_encoder = config.getoption("freeze_unet_encoder", True)
        seed = config.getoption("seed", 1123)

        inst = cls(
            config_path=config_path,
            text_config_path=text_config_path,
            vae_config_path=vae_config_path,
            controlnet_config_path=controlnet_config_path,
            scheduler_config_path=scheduler_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            freeze_vae_encoder=freeze_vae_encoder,
            freeze_text_encoder=freeze_text_encoder,
            freeze_unet_encoder=freeze_unet_encoder,
            seed=seed,
        )

        weight_path = config.getoption("pretrained_weight_path", None)

        state_dict = None
        if weight_path is None and pretrain_infos is not None:
            weight_path = [
                cached_path(nested_dict_value(pretrain_infos, "unet", "weight")),
                cached_path(nested_dict_value(pretrain_infos, "text", "weight")),
                cached_path(nested_dict_value(pretrain_infos, "vae", "weight")),
                # cached_path(nested_dict_value(pretrain_infos, "controlnet", "weight")),
            ]
            controlnet = cached_path(nested_dict_value(pretrain_infos, "controlnet", "weight"))
            state_dict = torch.load(controlnet, map_location="cpu")
            state_dict = {f"controlnet.{k}": v for k, v in state_dict.items()} 


        if weight_path is not None:
            inst.from_pretrained(weight_path, state_dict=state_dict)
        
        return inst

    @autocast()
    def forward(
        self,
        input_ids: torch.Tensor,
        pixel_values: torch.Tensor,
        condition_pixel_values: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
    ):
        outputs = super().forward(
            pixel_values=pixel_values,
            condition_pixel_values=condition_pixel_values,
            input_ids=input_ids,
            attention_mask=attention_mask,
        )
        return LossOutputs(loss=outputs)

    @add_default_section_for_function("core/model/diffusion/controlnet")
    @autocast()
    def generate(
        self,
        input_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        condition_pixel_values: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        guidance_scale: Optional[float] = 7.5,
    ):
        outputs = super().generate(
            input_ids=input_ids,
            negative_input_ids=negative_input_ids,
            condition_pixel_values=condition_pixel_values,
            attention_mask=attention_mask,
            negative_attention_mask=negative_attention_mask,
            guidance_scale=guidance_scale,
        )

        return DiffusionOutputs(outputs=outputs.images)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
import torch.nn as nn
from dataclasses import dataclass
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.cli import (
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli import WriterMixin, WriterOutputs
from unitorch.cli.models.modeling_utils import ListTensorsOutputs, ListTensorsTargets


@dataclass
class SegmentationOutputs(ListTensorsOutputs, WriterMixin):
    outputs: List[torch.Tensor]


@dataclass
class SegmentationTargets(ListTensorsTargets):
    targets: List[torch.Tensor]
    sample_weight: Optional[torch.Tensor] = torch.empty(0)


class SegmentationProcessor:
    def __init__(
        self,
    ):
        pass

    @classmethod
    @add_default_section_for_init("core/process/segmentation")
    def from_core_configure(cls, config, **kwargs):
        pass

    @register_process("core/postprocess/segmentation")
    def _segmentation(
        self,
        outputs: SegmentationOutputs,
    ):
        results = outputs.to_pandas()
        assert results.shape[0] == 0 or results.shape[0] == len(outputs.outputs)
        results["pixel_class"] = [
            m.numpy().argmax(-1).reshape(-1).tolist() for m in outputs.outputs
        ]
        return WriterOutputs(results)


def segmentation_model_decorator(cls):
    class SegmentationModel(nn.Module):
        def __init__(self, *args, **kwargs):
            super().__init__()
            if "__segmentation_model__" in kwargs:
                self.model = kwargs.pop("__segmentation_model__")
            else:
                self.model = cls(*args, **kwargs)

            __more_attrs__ = [
                "load_state_dict",
                "state_dict",
                "save_checkpoint",
                "from_checkpoint",
                "from_pretrained",
            ]
            for __more_attr__ in __more_attrs__:
                setattr(self, __more_attr__, getattr(self.model, __more_attr__))

        def forward(self, *args, **kwargs):
            if self.training:
                return self.model(*args, **kwargs)
            return self.model.segment(*args, **kwargs)

        @classmethod
        def from_core_configure(_cls, cfg, **kwargs):
            model = cls.from_core_configure(cfg, **kwargs)
            return _cls(__segmentation_model__=model)

    return SegmentationModel



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.loss import (
    CELoss as _CELoss,
    BCELoss as _BCELoss,
    LMLoss as _LMLoss,
    MSELoss as _MSELoss,
    ProphetnetLoss as _ProphetnetLoss,
)
from unitorch.cli import add_default_section_for_init, register_loss
from unitorch.cli.models import (
    ClassificationOutputs,
    ClassificationTargets,
    GenerationOutputs,
    GenerationTargets,
)
import unitorch.cli.loss.ranking


@register_loss("core/loss/ce")
class CELoss(_CELoss):
    def __init__(
        self,
        smoothing_alpha: Optional[float] = 0.0,
    ):
        super().__init__(
            smoothing_alpha,
        )

    @classmethod
    @add_default_section_for_init("core/loss/ce")
    def from_core_configure(cls, config, **kwargs):
        pass

    def forward(
        self,
        outputs: ClassificationOutputs,
        targets: ClassificationTargets,
    ):
        outputs = outputs.outputs
        weights = targets.sample_weight
        targets = targets.targets

        return super().forward(
            input=outputs,
            target=targets,
            sample_weight=weights if weights.numel() > 0 else None,
        )


@register_loss("core/loss/bce")
class BCELoss(_BCELoss):
    def __init__(
        self,
    ):
        super().__init__()

    @classmethod
    @add_default_section_for_init("core/loss/bce")
    def from_core_configure(cls, config, **kwargs):
        pass

    def forward(
        self,
        outputs: ClassificationOutputs,
        targets: ClassificationTargets,
    ):
        outputs = outputs.outputs
        weights = targets.sample_weight
        targets = targets.targets

        return super().forward(
            input=outputs,
            target=targets,
            sample_weight=weights if weights.numel() > 0 else None,
        )


@register_loss("core/loss/lm")
class LMLoss(_LMLoss):
    def __init__(
        self,
    ):
        super().__init__()

    @classmethod
    @add_default_section_for_init("core/loss/lm")
    def from_core_configure(cls, config, **kwargs):
        pass

    def forward(
        self,
        outputs: GenerationOutputs,
        targets: GenerationTargets,
    ):
        outputs = outputs.sequences
        masks = targets.masks
        weights = targets.sample_weight
        targets = targets.refs

        return super().forward(
            input=outputs,
            target=targets,
            masks=masks if masks.numel() > 0 else None,
            sample_weight=weights if weights.numel() > 0 else None,
        )


@register_loss("core/loss/mse")
class MSELoss(_MSELoss):
    def __init__(
        self,
    ):
        super().__init__()

    @classmethod
    @add_default_section_for_init("core/loss/mse")
    def from_core_configure(cls, config, **kwargs):
        pass

    def forward(
        self,
        outputs: ClassificationOutputs,
        targets: ClassificationTargets,
    ):
        outputs = outputs.outputs
        weights = targets.sample_weight
        targets = targets.targets

        return super().forward(
            input=outputs,
            target=targets,
            sample_weight=weights if weights.numel() > 0 else None,
        )


@register_loss("core/loss/prophetnet")
class ProphetnetLoss(_ProphetnetLoss):
    def __init__(
        self,
    ):
        super().__init__()

    @classmethod
    @add_default_section_for_init("core/loss/prophetnet")
    def from_core_configure(cls, config, **kwargs):
        pass

    def forward(
        self,
        outputs: GenerationOutputs,
        targets: GenerationTargets,
    ):
        outputs = outputs.sequences
        masks = targets.masks
        weights = targets.sample_weight
        targets = targets.refs

        return super().forward(
            input=outputs,
            target=targets,
            masks=masks if masks.numel() > 0 else None,
            sample_weight=weights if weights.numel() > 0 else None,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch import is_diffusers_available

if is_diffusers_available():

    from unitorch.models.diffusers.modeling_controlnet import (
        ControlNetForImageGeneration,
    )
    from unitorch.models.diffusers.processing_controlnet import ControlNetProcessor
    from unitorch.models.diffusers.processing_stable import StableProcessor
    from unitorch.models.diffusers.modeling_stable import (
        StableForImageGeneration,
        StableForText2ImageGeneration,
    )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import warnings
import logging
import torch
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from peft import (
    PeftConfig,
    PeftType,
    PromptLearningConfig,
    PeftModelForSequenceClassification,
)
from unitorch.utils import replace


@replace(PeftModelForSequenceClassification)
class PeftModelForSequenceClassification(PeftModelForSequenceClassification):
    def __init__(self, model, peft_config: PeftConfig, adapter_name="default"):
        super().__init__(model, peft_config, adapter_name)

    def forward(
        self,
        input_ids=None,
        attention_mask=None,
        inputs_embeds=None,
        output_attentions=None,
        output_hidden_states=None,
        return_dict=None,
        **kwargs,
    ):
        return_dict = (
            return_dict if return_dict is not None else self.config.use_return_dict
        )
        peft_config = self.active_peft_config
        if not isinstance(peft_config, PromptLearningConfig):
            return self.base_model(
                input_ids=input_ids,
                attention_mask=attention_mask,
                inputs_embeds=inputs_embeds,
                output_attentions=output_attentions,
                output_hidden_states=output_hidden_states,
                return_dict=return_dict,
                **kwargs,
            )

        labels = kwargs.pop("labels", None)

        batch_size = input_ids.shape[0]
        if attention_mask is not None:
            # concat prompt attention mask
            prefix_attention_mask = torch.ones(
                batch_size, peft_config.num_virtual_tokens
            ).to(self.device)
            attention_mask = torch.cat((prefix_attention_mask, attention_mask), dim=1)
        if kwargs.get("position_ids", None) is not None:
            warnings.warn(
                "Position ids are not supported for parameter efficient tuning. Ignoring position ids."
            )
            kwargs["position_ids"] = None
        kwargs.update(
            {
                "attention_mask": attention_mask,
                "labels": labels,
                "output_attentions": output_attentions,
                "output_hidden_states": output_hidden_states,
                "return_dict": return_dict,
            }
        )

        if peft_config.peft_type == PeftType.PREFIX_TUNING:
            return self._prefix_tuning_forward(input_ids=input_ids, **kwargs)
        else:
            if kwargs.get("token_type_ids", None) is not None:
                kwargs["token_type_ids"] = torch.cat(
                    (
                        torch.zeros(batch_size, peft_config.num_virtual_tokens).to(
                            self.device
                        ),
                        kwargs["token_type_ids"],
                    ),
                    dim=1,
                ).long()
            if inputs_embeds is None:
                inputs_embeds = self.word_embeddings(input_ids)
            prompts = self.get_prompt(batch_size=batch_size)
            prompts = prompts.to(inputs_embeds.dtype)
            inputs_embeds = torch.cat((prompts, inputs_embeds), dim=1)
            return self.base_model(inputs_embeds=inputs_embeds, **kwargs)


class PeftCheckpointMixin:
    checkpoint_name = "pytorch_peft_model.bin"

    def save_checkpoint(
        self,
        ckpt_dir: str,
        weight_name: str = None,
        **kwargs,
    ):
        """
        Save the model's current state as a checkpoint.

        Args:
            ckpt_dir (str): Directory path to save the checkpoint.
            weight_name (str): Name of the weight file.

        Returns:
            None
        """
        if weight_name is None:
            weight_name = self.checkpoint_name
        state_dict = self.state_dict()
        state_dict = {k: v for k, v in state_dict.items() if ".lora_" in k}
        weight_path = os.path.join(ckpt_dir, weight_name)
        torch.save(state_dict, weight_path)
        logging.info(f"{type(self).__name__} model save checkpoint to {weight_path}")

    def from_checkpoint(
        self,
        ckpt_dir: str,
        weight_name: Optional[str] = None,
        **kwargs,
    ):
        """
        Load model weights from a checkpoint.

        Args:
            ckpt_dir (str): Directory path of the checkpoint.
            weight_name (str): Name of the weight file.

        Returns:
            None
        """
        if weight_name is None:
            weight_name = self.checkpoint_name
        weight_path = os.path.join(ckpt_dir, weight_name)
        if not os.path.exists(weight_path):
            return
        state_dict = torch.load(weight_path, map_location="cpu")
        self.load_state_dict(state_dict, strict=False)
        logging.info(
            f"{type(self).__name__} model load weight from checkpoint {weight_path}"
        )


from unitorch.models.peft.modeling_bloom_adalora import (
    BloomAdaLoraForClassification,
    BloomAdaLoraForGeneration,
)
from unitorch.models.peft.modeling_bloom_lora import (
    BloomLoraForClassification,
    BloomLoraForGeneration,
)
from unitorch.models.peft.modeling_llama_adalora import (
    LlamaAdaLoraForClassification,
    LlamaAdaLoraForGeneration,
)
from unitorch.models.peft.modeling_llama_lora import (
    LlamaLoraForClassification,
    LlamaLoraForGeneration,
)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import sys
import fire
import atexit
import signal
import time
import logging
import torch
import importlib
import unitorch.cli
import unitorch.cli.services
from pathlib import Path
from unitorch.utils import ActiveGPUJob
from unitorch.cli import CoreConfigureParser
from unitorch.cli import (
    import_library,
    cached_path,
    registered_service,
    init_registered_module,
)


def _sigterm_handler(signo, frame):
    raise SystemExit(1)


def daemonize(pid_file, name):
    if os.path.exists(pid_file):
        raise RuntimeError(f"unitorch-service {name} already running")
    pid = os.fork()
    if pid > 0:
        # sys.exit(0)
        os._exit(0)

    os.chdir("/")
    os.setsid()
    os.umask(0)

    _pid = os.fork()
    if _pid:
        # sys.exit(0)
        os._exit(0)

    sys.stdout.flush()
    sys.stderr.flush()

    stdin_file = f"/tmp/unitorch_service_{name}.stdin.log"
    stdout_file = f"/tmp/unitorch_service_{name}.stdout.log"

    Path(stdin_file).touch()
    Path(stdout_file).touch()

    with open(stdin_file, "rb") as read_file, open(stdout_file, "ab") as write_file:
        os.dup2(read_file.fileno(), sys.stdin.fileno())
        os.dup2(write_file.fileno(), sys.stdout.fileno())
        os.dup2(write_file.fileno(), sys.stderr.fileno())

    with open(pid_file, "w") as f:
        f.write(str(os.getpid()))

    atexit.register(lambda: os.remove(pid_file))
    signal.signal(signal.SIGTERM, _sigterm_handler)


def gpu(pid):
    tensor = torch.rand(10240, 10240).cuda()
    while True:
        _ = torch.matmul(tensor, tensor)
        time.sleep(1)


def start(name, inst, daemon_mode):
    name = name.replace("/", "_")
    pid_file = f"/tmp/unitorch_service_{name}.pid"
    if daemon_mode:
        daemonize(pid_file, name)
    inst.start()


def stop(name, inst):
    name = name.replace("/", "_")
    pid_file = f"/tmp/unitorch_service_{name}.pid"
    if os.path.exists(pid_file):
        with open(pid_file) as f:
            os.kill(int(f.read()), signal.SIGTERM)


def restart(name, inst):
    stop(name, inst)
    start(name, inst)


@fire.decorators.SetParseFn(str)
def service(service_action: str, service_path_or_dir: str, **kwargs):
    config_file = kwargs.pop("config_file", "config.ini")

    if service_path_or_dir and os.path.isdir(service_path_or_dir):
        config_path = os.path.join(service_path_or_dir, config_file)
        sys.path.insert(0, service_path_or_dir)
        for f in os.listdir(service_path_or_dir):
            fpath = os.path.normpath(os.path.join(service_path_or_dir, f))
            if (
                not f.startswith("_")
                and not f.startswith(".")
                and (f.endswith(".py") or os.path.isdir(fpath))
            ):
                fname = f[:-3] if f.endswith(".py") else f
                module = importlib.import_module(f"{fname}")
    else:
        config_path = cached_path(service_path_or_dir)

    params = []
    for k, v in kwargs.items():
        if k.count("@") > 0:
            k0 = k.split("@")[0]
            k1 = "@".join(k.split("@")[1:])
        else:
            k0 = "core/cli"
            k1 = k
        params.append((k0, k1, v))

    if config_path is not None:
        config = CoreConfigureParser(config_path, params=params)
    else:
        config = CoreConfigureParser(params=params)

    depends_libraries = config.getdefault("core/cli", "depends_libraries", None)

    if depends_libraries:
        for library in depends_libraries:
            import_library(library)

    daemon_mode = config.getdefault("core/cli", "daemon_mode", True)
    gpu_mode = config.getdefault("core/cli", "gpu_mode", False)
    service_name = config.getdefault("core/cli", "service_name", None)
    assert service_name is not None
    main_service_cls = registered_service.get(service_name)
    if main_service_cls is None:
        raise ValueError(f"service {service_name} not found")

    if gpu_mode:
        with ActiveGPUJob() as _:
            service_inst = main_service_cls["obj"](config)
    elif service_action in ["start", "restart"]:
        service_inst = main_service_cls["obj"](config)
    else:
        service_inst = None

    hexsha = config.hexsha(6)
    service_name = service_name + f"@{hexsha}"
    if service_action == "start":
        start(service_name, service_inst, daemon_mode)
    elif service_action == "stop":
        stop(service_name, service_inst)
    elif service_action == "restart":
        restart(service_name, service_inst)
    else:
        raise ValueError(f"service action {service_action} not found")


def cli_main():
    fire.Fire(service)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
from functools import partial

import torch
from transformers import ProphetNetTokenizer
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.models import HfTextGenerationProcessor


def get_prophetnet_tokenizer(vocab_path, special_input_ids=dict()):
    """
    Get the tokenizer for ProphetNet model.

    Args:
        vocab_path (str): The path to the vocabulary file.
        special_input_ids (Dict, optional): Special input IDs mapping. Defaults to an empty dictionary.

    Returns:
        ProphetNetTokenizer: The tokenizer object.
    """
    assert os.path.exists(vocab_path)
    tokenizer = ProphetNetTokenizer(vocab_path)
    for token, _id in special_input_ids.items():
        tokenizer.added_tokens_encoder[token] = _id
        tokenizer.unique_no_split_tokens.append(token)
        tokenizer.added_tokens_decoder[_id] = token
        tokenizer.add_tokens(token, special_tokens=True)
    return tokenizer


class ProphetNetProcessor(HfTextGenerationProcessor):
    """
    Processor for ProphetNet-based text generation models.
    """

    def __init__(
        self,
        vocab_path: str,
        special_input_ids: Optional[Dict] = dict(),
        max_seq_length: Optional[int] = 128,
        max_gen_seq_length: Optional[int] = 48,
    ):
        """
        Initializes the ProphetNetProcessor.

        Args:
            vocab_path (str): Path to the vocabulary file.
            special_input_ids (Optional[Dict]): Special input IDs. Defaults to an empty dictionary.
            max_seq_length (Optional[int]): Maximum sequence length. Defaults to 128.
            max_gen_seq_length (Optional[int]): Maximum generated sequence length. Defaults to 48.
        """
        tokenizer = get_prophetnet_tokenizer(
            vocab_path,
            special_input_ids=special_input_ids,
        )
        super().__init__(
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.
""" Wrapper for dyhead cuda extension """
import torch
import math
import torch.nn as nn
from torch.nn.modules.utils import _pair
from torch.autograd import Function
from torch.autograd.function import once_differentiable

try:
    import unitorch.clib.dyhead as dyhead

    is_kernel_available = True
except:
    is_kernel_available = False


class DeformConvFunction(Function):
    @staticmethod
    def forward(
        ctx,
        input,
        offset,
        weight,
        stride=1,
        padding=0,
        dilation=1,
        groups=1,
        deformable_groups=1,
        im2col_step=64,
    ):
        if input is not None and input.dim() != 4:
            raise ValueError(
                "Expected 4D tensor as input, got {}D tensor instead.".format(
                    input.dim()
                )
            )
        ctx.stride = _pair(stride)
        ctx.padding = _pair(padding)
        ctx.dilation = _pair(dilation)
        ctx.groups = groups
        ctx.deformable_groups = deformable_groups
        ctx.im2col_step = im2col_step

        ctx.save_for_backward(input, offset, weight)

        output = input.new_empty(
            DeformConvFunction._output_size(
                input,
                weight,
                ctx.padding,
                ctx.dilation,
                ctx.stride,
            )
        )

        ctx.bufs_ = [input.new_empty(0), input.new_empty(0)]  # columns, ones

        if not input.is_cuda:
            raise NotImplementedError
        else:
            cur_im2col_step = min(ctx.im2col_step, input.shape[0])
            assert (
                input.shape[0] % cur_im2col_step
            ) == 0, "im2col step must divide batchsize"
            dyhead.deform_conv_forward(
                input,
                weight,
                offset,
                output,
                ctx.bufs_[0],
                ctx.bufs_[1],
                weight.size(3),
                weight.size(2),
                ctx.stride[1],
                ctx.stride[0],
                ctx.padding[1],
                ctx.padding[0],
                ctx.dilation[1],
                ctx.dilation[0],
                ctx.groups,
                ctx.deformable_groups,
                cur_im2col_step,
            )
        return output

    @staticmethod
    @once_differentiable
    def backward(ctx, grad_output):
        input, offset, weight = ctx.saved_tensors

        grad_input = grad_offset = grad_weight = None

        if not grad_output.is_cuda:
            raise NotImplementedError
        else:
            cur_im2col_step = min(ctx.im2col_step, input.shape[0])
            assert (
                input.shape[0] % cur_im2col_step
            ) == 0, "im2col step must divide batchsize"

            if ctx.needs_input_grad[0] or ctx.needs_input_grad[1]:
                grad_input = torch.zeros_like(input)
                grad_offset = torch.zeros_like(offset)
                dyhead.deform_conv_backward_input(
                    input,
                    offset,
                    grad_output,
                    grad_input,
                    grad_offset,
                    weight,
                    ctx.bufs_[0],
                    weight.size(3),
                    weight.size(2),
                    ctx.stride[1],
                    ctx.stride[0],
                    ctx.padding[1],
                    ctx.padding[0],
                    ctx.dilation[1],
                    ctx.dilation[0],
                    ctx.groups,
                    ctx.deformable_groups,
                    cur_im2col_step,
                )

            if ctx.needs_input_grad[2]:
                grad_weight = torch.zeros_like(weight)
                dyhead.deform_conv_backward_parameters(
                    input,
                    offset,
                    grad_output,
                    grad_weight,
                    ctx.bufs_[0],
                    ctx.bufs_[1],
                    weight.size(3),
                    weight.size(2),
                    ctx.stride[1],
                    ctx.stride[0],
                    ctx.padding[1],
                    ctx.padding[0],
                    ctx.dilation[1],
                    ctx.dilation[0],
                    ctx.groups,
                    ctx.deformable_groups,
                    1,
                    cur_im2col_step,
                )

        return (
            grad_input,
            grad_offset,
            grad_weight,
            None,
            None,
            None,
            None,
            None,
        )

    @staticmethod
    def _output_size(input, weight, padding, dilation, stride):
        channels = weight.size(0)
        output_size = (input.size(0), channels)
        for d in range(input.dim() - 2):
            in_size = input.size(d + 2)
            pad = padding[d]
            kernel = dilation[d] * (weight.size(d + 2) - 1) + 1
            stride_ = stride[d]
            output_size += ((in_size + (2 * pad) - kernel) // stride_ + 1,)
        if not all(map(lambda s: s > 0, output_size)):
            raise ValueError(
                "convolution input is too small (output would be {})".format(
                    "x".join(map(str, output_size))
                )
            )
        return output_size


class ModulatedDeformConvFunction(Function):
    @staticmethod
    def forward(
        ctx,
        input,
        offset,
        mask,
        weight,
        bias=None,
        stride=1,
        padding=0,
        dilation=1,
        groups=1,
        deformable_groups=1,
    ):
        ctx.stride = stride
        ctx.padding = padding
        ctx.dilation = dilation
        ctx.groups = groups
        ctx.deformable_groups = deformable_groups
        ctx.with_bias = bias is not None
        if not ctx.with_bias:
            bias = input.new_empty(1)  # fake tensor
        if not input.is_cuda:
            raise NotImplementedError
        if (
            weight.requires_grad
            or mask.requires_grad
            or offset.requires_grad
            or input.requires_grad
        ):
            ctx.save_for_backward(input, offset, mask, weight, bias)
        output = input.new_empty(
            ModulatedDeformConvFunction._infer_shape(ctx, input, weight)
        )
        ctx._bufs = [input.new_empty(0), input.new_empty(0)]
        dyhead.modulated_deform_conv_forward(
            input,
            weight,
            bias,
            ctx._bufs[0],
            offset,
            mask,
            output,
            ctx._bufs[1],
            weight.shape[2],
            weight.shape[3],
            ctx.stride,
            ctx.stride,
            ctx.padding,
            ctx.padding,
            ctx.dilation,
            ctx.dilation,
            ctx.groups,
            ctx.deformable_groups,
            ctx.with_bias,
        )
        return output

    @staticmethod
    @once_differentiable
    def backward(ctx, grad_output):
        if not grad_output.is_cuda:
            raise NotImplementedError
        input, offset, mask, weight, bias = ctx.saved_tensors
        grad_input = torch.zeros_like(input)
        grad_offset = torch.zeros_like(offset)
        grad_mask = torch.zeros_like(mask)
        grad_weight = torch.zeros_like(weight)
        grad_bias = torch.zeros_like(bias)
        dyhead.modulated_deform_conv_backward(
            input,
            weight,
            bias,
            ctx._bufs[0],
            offset,
            mask,
            ctx._bufs[1],
            grad_input,
            grad_weight,
            grad_bias,
            grad_offset,
            grad_mask,
            grad_output,
            weight.shape[2],
            weight.shape[3],
            ctx.stride,
            ctx.stride,
            ctx.padding,
            ctx.padding,
            ctx.dilation,
            ctx.dilation,
            ctx.groups,
            ctx.deformable_groups,
            ctx.with_bias,
        )
        if not ctx.with_bias:
            grad_bias = None

        return (
            grad_input,
            grad_offset,
            grad_mask,
            grad_weight,
            grad_bias,
            None,
            None,
            None,
            None,
            None,
        )

    @staticmethod
    def _infer_shape(ctx, input, weight):
        n = input.size(0)
        channels_out = weight.size(0)
        height, width = input.shape[2:4]
        kernel_h, kernel_w = weight.shape[2:4]
        height_out = (
            height + 2 * ctx.padding - (ctx.dilation * (kernel_h - 1) + 1)
        ) // ctx.stride + 1
        width_out = (
            width + 2 * ctx.padding - (ctx.dilation * (kernel_w - 1) + 1)
        ) // ctx.stride + 1
        return n, channels_out, height_out, width_out


deform_conv = DeformConvFunction.apply
modulated_deform_conv = ModulatedDeformConvFunction.apply


class DeformConv(nn.Module):
    def __init__(
        self,
        in_channels,
        out_channels,
        kernel_size,
        stride=1,
        padding=0,
        dilation=1,
        groups=1,
        deformable_groups=1,
        bias=False,
    ):
        assert not bias
        super().__init__()
        self.with_bias = bias

        assert (
            in_channels % groups == 0
        ), "in_channels {} cannot be divisible by groups {}".format(in_channels, groups)
        assert (
            out_channels % groups == 0
        ), "out_channels {} cannot be divisible by groups {}".format(
            out_channels, groups
        )
        self.in_channels = in_channels
        self.out_channels = out_channels
        self.kernel_size = _pair(kernel_size)
        self.stride = _pair(stride)
        self.padding = _pair(padding)
        self.dilation = _pair(dilation)
        self.groups = groups
        self.deformable_groups = deformable_groups

        self.weight = nn.Parameter(
            torch.Tensor(
                out_channels,
                in_channels // self.groups,
                *self.kernel_size,
            )
        )

        self.reset_parameters()

    def reset_parameters(self):
        n = self.in_channels
        for k in self.kernel_size:
            n *= k
        stdv = 1.0 / math.sqrt(n)
        self.weight.data.uniform_(-stdv, stdv)

    def forward(self, input, offset):
        return deform_conv(
            input,
            offset,
            self.weight,
            self.stride,
            self.padding,
            self.dilation,
            self.groups,
            self.deformable_groups,
        )

    def __repr__(self):
        return "".join(
            [
                "{}(".format(self.__class__.__name__),
                "in_channels={}, ".format(self.in_channels),
                "out_channels={}, ".format(self.out_channels),
                "kernel_size={}, ".format(self.kernel_size),
                "stride={}, ".format(self.stride),
                "dilation={}, ".format(self.dilation),
                "padding={}, ".format(self.padding),
                "groups={}, ".format(self.groups),
                "deformable_groups={}, ".format(self.deformable_groups),
                "bias={})".format(self.with_bias),
            ]
        )


class ModulatedDeformConv(nn.Module):
    def __init__(
        self,
        in_channels,
        out_channels,
        kernel_size,
        stride=1,
        padding=0,
        dilation=1,
        groups=1,
        deformable_groups=1,
        bias=True,
    ):
        super().__init__()
        self.in_channels = in_channels
        self.out_channels = out_channels
        self.kernel_size = _pair(kernel_size)
        self.stride = stride
        self.padding = padding
        self.dilation = dilation
        self.groups = groups
        self.deformable_groups = deformable_groups
        self.with_bias = bias

        self.weight = nn.Parameter(
            torch.Tensor(out_channels, in_channels // groups, *self.kernel_size)
        )
        if bias:
            self.bias = nn.Parameter(torch.Tensor(out_channels))
        else:
            self.register_parameter("bias", None)
        self.reset_parameters()

    def reset_parameters(self):
        n = self.in_channels
        for k in self.kernel_size:
            n *= k
        stdv = 1.0 / math.sqrt(n)
        self.weight.data.uniform_(-stdv, stdv)
        if self.bias is not None:
            self.bias.data.zero_()

    def forward(self, input, offset, mask):
        return modulated_deform_conv(
            input,
            offset,
            mask,
            self.weight,
            self.bias,
            self.stride,
            self.padding,
            self.dilation,
            self.groups,
            self.deformable_groups,
        )

    def __repr__(self):
        return "".join(
            [
                "{}(".format(self.__class__.__name__),
                "in_channels={}, ".format(self.in_channels),
                "out_channels={}, ".format(self.out_channels),
                "kernel_size={}, ".format(self.kernel_size),
                "stride={}, ".format(self.stride),
                "dilation={}, ".format(self.dilation),
                "padding={}, ".format(self.padding),
                "groups={}, ".format(self.groups),
                "deformable_groups={}, ".format(self.deformable_groups),
                "bias={})".format(self.with_bias),
            ]
        )


class ModulatedDeformConvPack(ModulatedDeformConv):
    def __init__(
        self,
        in_channels,
        out_channels,
        kernel_size,
        stride=1,
        padding=0,
        dilation=1,
        groups=1,
        deformable_groups=1,
        bias=True,
    ):
        super().__init__(
            in_channels,
            out_channels,
            kernel_size,
            stride,
            padding,
            dilation,
            groups,
            deformable_groups,
            bias,
        )

        self.conv_offset_mask = nn.Conv2d(
            self.in_channels // self.groups,
            self.deformable_groups * 3 * self.kernel_size[0] * self.kernel_size[1],
            kernel_size=self.kernel_size,
            stride=_pair(self.stride),
            padding=_pair(self.padding),
            bias=True,
        )
        self.init_offset()

    def init_offset(self):
        self.conv_offset_mask.weight.data.zero_()
        self.conv_offset_mask.bias.data.zero_()

    def forward(self, input):
        out = self.conv_offset_mask(input)
        o1, o2, mask = torch.chunk(out, 3, dim=1)
        offset = torch.cat((o1, o2), dim=1)
        mask = torch.sigmoid(mask)
        return modulated_deform_conv(
            input,
            offset,
            mask,
            self.weight,
            self.bias,
            self.stride,
            self.padding,
            self.dilation,
            self.groups,
            self.deformable_groups,
        )


# Sigmoid Focal Loss
class SigmoidFocalLossFunction(Function):
    @staticmethod
    def forward(ctx, logits, targets, gamma, alpha):
        ctx.save_for_backward(logits, targets)
        num_classes = logits.shape[1]
        ctx.num_classes = num_classes
        ctx.gamma = gamma
        ctx.alpha = alpha

        losses = dyhead.sigmoid_focal_loss_forward(
            logits,
            targets,
            num_classes,
            gamma,
            alpha,
        )
        return losses

    @staticmethod
    @once_differentiable
    def backward(ctx, d_loss):
        logits, targets = ctx.saved_tensors
        num_classes = ctx.num_classes
        gamma = ctx.gamma
        alpha = ctx.alpha
        d_loss = d_loss.contiguous()
        d_logits = dyhead.sigmoid_focal_loss_backward(
            logits,
            targets,
            d_loss,
            num_classes,
            gamma,
            alpha,
        )
        return d_logits, None, None, None, None


sigmoid_focal_loss_cuda = SigmoidFocalLossFunction.apply


class SigmoidFocalLoss(nn.Module):
    def __init__(self, gamma, alpha, reduction="sum"):
        super().__init__()
        self.gamma = gamma
        self.alpha = alpha
        self.reduction = reduction

    def forward(self, logits, targets):
        loss = sigmoid_focal_loss_cuda(
            logits.to(torch.float32), targets.to(torch.int32), self.gamma, self.alpha
        )
        if self.reduction == "mean":
            return loss.mean()
        if self.reduction == "sum":
            return loss.sum()
        return loss

    def __repr__(self):
        tmpstr = self.__class__.__name__ + "("
        tmpstr += "gamma=" + str(self.gamma)
        tmpstr += ", alpha=" + str(self.alpha)
        tmpstr += ")"
        return tmpstr



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import logging
import torch
import torch.nn as nn
import torch.nn.functional as F
import transformers
from copy import deepcopy
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers import T5Config, T5Model, T5ForConditionalGeneration
from transformers.models.t5.modeling_t5 import T5Stack

from unitorch.utils.decorators import replace
from unitorch.models import GenericModel, GenericOutputs


@replace(transformers.models.t5.modeling_t5.T5Attention)
class T5AttentionV2(transformers.models.t5.modeling_t5.T5Attention):
    def __init__(
        self,
        config: T5Config,
        has_relative_attention_bias=False,
    ):
        super().__init__(
            config=config,
            has_relative_attention_bias=has_relative_attention_bias,
        )

    def forward(
        self,
        hidden_states,
        mask=None,
        key_value_states=None,
        position_bias=None,
        past_key_value=None,
        layer_head_mask=None,
        query_length=None,
        use_cache=False,
        output_attentions=False,
    ):
        """
        Self-attention (if key_value_states is None) or attention over source sentence (provided by key_value_states).
        """
        # Input is (batch_size, seq_length, dim)
        # Mask is (batch_size, key_length) (non-causal) or (batch_size, key_length, key_length)
        # past_key_value[0] is (batch_size, n_heads, q_len - 1, dim_per_head)
        batch_size, seq_length = hidden_states.shape[:2]

        real_seq_length = seq_length

        if past_key_value is not None:
            assert (
                len(past_key_value) == 2
            ), f"past_key_value should have 2 past states: keys and values. Got { len(past_key_value)} past states"
            real_seq_length += (
                past_key_value[0].shape[2] if query_length is None else query_length
            )

        key_length = (
            real_seq_length if key_value_states is None else key_value_states.shape[1]
        )

        def shape(states):
            """projection"""
            return states.view(
                states.size()[0], -1, self.n_heads, self.key_value_proj_dim
            ).transpose(1, 2)

        def unshape(states):
            """reshape"""
            return (
                states.transpose(1, 2).contiguous().view(batch_size, -1, self.inner_dim)
            )

        def project(hidden_states, proj_layer, key_value_states, past_key_value):
            """projects hidden states correctly to key/query states"""
            if key_value_states is None:
                # self-attn
                # (batch_size, n_heads, seq_length, dim_per_head)
                hidden_states = shape(proj_layer(hidden_states))
            elif past_key_value is None:
                # cross-attn
                # (batch_size, n_heads, seq_length, dim_per_head)
                hidden_states = shape(proj_layer(key_value_states))

            if past_key_value is not None:
                if key_value_states is None:
                    # self-attn
                    # (batch_size, n_heads, key_length, dim_per_head)
                    hidden_states = torch.cat([past_key_value, hidden_states], dim=2)
                elif past_key_value.shape[2] != key_value_states.shape[1]:
                    # checking that the `sequence_length` of the `past_key_value` is the same as
                    # the provided `key_value_states` to support prefix tuning
                    # cross-attn
                    # (batch_size, n_heads, seq_length, dim_per_head)
                    hidden_states = shape(proj_layer(key_value_states))
                else:
                    # cross-attn
                    hidden_states = past_key_value
            return hidden_states

        # get query states
        query_states = shape(
            self.q(hidden_states)
        )  # (batch_size, n_heads, seq_length, dim_per_head)

        # get key/value states
        key_states = project(
            hidden_states,
            self.k,
            key_value_states,
            past_key_value[0] if past_key_value is not None else None,
        )
        value_states = project(
            hidden_states,
            self.v,
            key_value_states,
            past_key_value[1] if past_key_value is not None else None,
        )

        kv_batch_size = key_states.size(0)

        # compute scores
        # scores = torch.matmul(
        #     query_states, key_states.transpose(3, 2)
        # )  # equivalent of torch.einsum("bnqd,bnkd->bnqk", query_states, key_states), compatible with onnx op>9
        scores = torch.einsum(
            "bxnqd,bnkd->bxnqk",
            query_states.view(kv_batch_size, -1, *query_states.size()[1:]),
            key_states,
        )
        scores = scores.reshape(-1, *scores.size()[2:])

        if position_bias is None:
            if not self.has_relative_attention_bias:
                position_bias = torch.zeros(
                    (1, self.n_heads, real_seq_length, key_length),
                    device=scores.device,
                    dtype=scores.dtype,
                )
                if self.gradient_checkpointing and self.training:
                    position_bias.requires_grad = True
            else:
                position_bias = self.compute_bias(
                    real_seq_length, key_length, device=scores.device
                )

            # if key and values are already calculated
            # we want only the last query position bias
            if past_key_value is not None:
                position_bias = position_bias[:, :, -hidden_states.size(1) :, :]

            if mask is not None:
                position_bias = (
                    position_bias + mask
                )  # (batch_size, n_heads, seq_length, key_length)

        if self.pruned_heads:
            mask = torch.ones(position_bias.shape[1])
            mask[list(self.pruned_heads)] = 0
            position_bias_masked = position_bias[:, mask.bool()]
        else:
            position_bias_masked = position_bias

        scores += position_bias_masked
        attn_weights = nn.functional.softmax(scores.float(), dim=-1).type_as(
            scores
        )  # (batch_size, n_heads, seq_length, key_length)
        attn_weights = nn.functional.dropout(
            attn_weights, p=self.dropout, training=self.training
        )  # (batch_size, n_heads, seq_length, key_length)

        # Mask heads if we want to
        if layer_head_mask is not None:
            attn_weights = attn_weights * layer_head_mask

        # attn_output = unshape(torch.matmul(attn_weights, value_states))  # (batch_size, seq_length, dim)
        attn_output = torch.einsum(
            "bxnqk,bnkd->bxnqd",
            attn_weights.view(kv_batch_size, -1, *attn_weights.size()[1:]),
            value_states,
        )
        attn_output = attn_output.reshape(-1, *attn_output.size()[2:])
        attn_output = unshape(attn_output)
        attn_output = self.o(attn_output)

        present_key_value_state = (
            (key_states, value_states) if (self.is_decoder and use_cache) else None
        )
        outputs = (attn_output,) + (present_key_value_state,) + (position_bias,)

        if output_attentions:
            outputs = outputs + (attn_weights,)
        return outputs


class T5ForGeneration(GenericModel):
    """
    T5 model for text generation tasks.
    """

    prefix_keys_in_state_dict = {"^(?!model\.).*": "model."}

    def __init__(
        self,
        config_path: str,
        gradient_checkpointing: Optional[bool] = False,
    ):
        """
        Initializes the T5ForGeneration model.

        Args:
            config_path (str): Path to the configuration file.
            gradient_checkpointing (Optional[bool]): Whether to use gradient checkpointing. Defaults to False.
        """
        super().__init__()
        self.config = T5Config.from_json_file(config_path)
        self.config.gradient_checkpointing = gradient_checkpointing
        self.model = T5ForConditionalGeneration(self.config)
        self.init_weights()

    def forward(
        self,
        input_ids: torch.Tensor,
        attention_mask: torch.Tensor,
        decoder_input_ids: torch.Tensor,
        decoder_attention_mask: torch.Tensor,
    ):
        """
        Performs forward pass of the T5ForGeneration model.

        Args:
            input_ids (torch.Tensor): Tensor of input token IDs.
            attention_mask (torch.Tensor): Tensor of attention mask.
            decoder_input_ids (torch.Tensor): Tensor of decoder input token IDs.
            decoder_attention_mask (torch.Tensor): Tensor of decoder attention mask.

        Returns:
            (torch.Tensor):The model's logits.
        """
        outputs = self.model(
            input_ids=input_ids,
            attention_mask=attention_mask,
            decoder_input_ids=decoder_input_ids,
            decoder_attention_mask=decoder_attention_mask,
            return_dict=True,
        )
        logits = outputs.logits
        return logits

    @torch.no_grad()
    def generate(
        self,
        input_ids: torch.Tensor,
        num_beams: Optional[int] = 5,
        decoder_start_token_id: Optional[int] = 0,
        decoder_end_token_id: Optional[int] = 1,
        num_return_sequences: Optional[int] = 1,
        min_gen_seq_length: Optional[int] = 0,
        max_gen_seq_length: Optional[int] = 48,
        repetition_penalty: Optional[float] = 1.0,
        no_repeat_ngram_size: Optional[int] = 0,
        early_stopping: Optional[bool] = True,
        length_penalty: Optional[float] = 1.0,
        num_beam_groups: Optional[int] = 1,
        diversity_penalty: Optional[float] = 0.0,
        do_sample: Optional[bool] = False,
        temperature: Optional[float] = 1.0,
        top_k: Optional[int] = 50,
        top_p: Optional[float] = 1.0,
    ):
        """
        Generates sequences using the T5ForGeneration model.

        Args:
            input_ids: The input token IDs.
            num_beams (int, optional): The number of beams for beam search. Defaults to 5.
            decoder_start_token_id (int, optional): The decoder's start token ID. Defaults to 2.
            decoder_end_token_id (int, optional): The decoder's end token ID. Defaults to 2.
            num_return_sequences (int, optional): The number of generated sequences to return. Defaults to 1.
            min_gen_seq_length (int, optional): The minimum length of the generated sequences. Defaults to 0.
            max_gen_seq_length (int, optional): The maximum length of the generated sequences. Defaults to 48.
            repetition_penalty (float, optional): The repetition penalty. Defaults to 1.0.
            no_repeat_ngram_size (int, optional): The size of n-grams to avoid repeating. Defaults to 0.
            early_stopping (bool, optional): Whether to stop generation early. Defaults to True.
            length_penalty (float, optional): The length penalty. Defaults to 1.0.
            num_beam_groups (int, optional): The number of beam groups for diverse beam search. Defaults to 1.
            diversity_penalty (float, optional): The diversity penalty. Defaults to 0.0.
            do_sample (bool, optional): Whether to use sampling for generation. Defaults to False.
            temperature (float, optional): The temperature for sampling. Defaults to 1.0.
            top_k (int, optional): The value for top-k sampling. Defaults to 50.
            top_p (float, optional): The value for top-p (nucleus) sampling. Defaults to 1.0.

        Returns:
            GenericOutputs: The generated sequences and their scores.
        """
        outputs = self.model.generate(
            input_ids,
            max_length=max_gen_seq_length,
            min_length=min_gen_seq_length,
            num_beams=num_beams,
            do_sample=do_sample,
            decoder_start_token_id=decoder_start_token_id,
            no_repeat_ngram_size=no_repeat_ngram_size,
            early_stopping=early_stopping,
            length_penalty=length_penalty,
            repetition_penalty=repetition_penalty,
            num_return_sequences=num_return_sequences,
            bos_token_id=decoder_start_token_id,
            eos_token_id=decoder_end_token_id,
            num_beam_groups=num_beam_groups,
            diversity_penalty=diversity_penalty,
            temperature=temperature,
            top_k=top_k,
            top_p=top_p,
            return_dict_in_generate=True,
            output_scores=True,
        )

        sequences = outputs.sequences.reshape(
            -1, num_return_sequences, outputs.sequences.size(-1)
        )
        outputs.sequences = torch.zeros(
            sequences.size(0), num_return_sequences, max_gen_seq_length
        ).to(device=sequences.device)
        outputs.sequences[:, :, : sequences.size(-1)].copy_(sequences)

        if num_return_sequences == 1:
            outputs.sequences = outputs.sequences.reshape(-1, max_gen_seq_length)

        return GenericOutputs(
            sequences=outputs.sequences,
            sequences_scores=outputs.sequences_scores,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from PIL import Image
from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
import unitorch.utils.palette


def make_grid(
    images: List[Image.Image], rows: int, cols: int, resize: Optional[List[int]] = None
) -> Image.Image:
    """
    Combines a list of images into a grid layout.

    Args:
        images (List[Image.Image]): List of PIL Images to combine into a grid.
        rows (int): Number of rows in the grid.
        cols (int): Number of columns in the grid.
        resize (Optional[List[int]], optional): Size to resize the images. Defaults to None.

    Returns:
        Image.Image: PIL Image object representing the grid layout.

    Raises:
        AssertionError: If the number of images is not equal to the product of rows and cols.
    """
    assert (
        len(images) == rows * cols
    ), "Number of images must be equal to the product of rows and cols."

    if resize is not None:
        w, h = resize
    else:
        w, h = images[0].size

    grid = Image.new("RGB", (w * cols, h * rows))
    for i, image in enumerate(images):
        grid.paste(image, box=(w * (i % cols), h * (i // cols)))

    return grid



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
import torch.nn as nn
from torch.optim import Adam, SGD
from transformers import AdamW
from unitorch.optim.lion import Lion



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

# pretrained infos
pretrained_llama_infos = {
    # llama
    "default-llama": {
        "config": "https://huggingface.co/huggyllama/llama-7b/resolve/main/config.json",
        "vocab": "https://huggingface.co/huggyllama/llama-7b/resolve/main/tokenizer.model",
    },
    "llama-7b-hf": {
        "config": "https://huggingface.co/decapoda-research/llama-7b-hf/resolve/main/config.json",
        "vocab": "https://huggingface.co/decapoda-research/llama-7b-hf/resolve/main/tokenizer.model",
        "weight": [
            f"https://huggingface.co/decapoda-research/llama-7b-hf/resolve/main/pytorch_model-{str(i).rjust(5, '0')}-of-00033.bin"
            for i in range(1, 34)
        ],
    },
    "llama-13b-hf": {
        "config": "https://huggingface.co/decapoda-research/llama-13b-hf/resolve/main/config.json",
        "vocab": "https://huggingface.co/decapoda-research/llama-13b-hf/resolve/main/tokenizer.model",
        "weight": [
            f"https://huggingface.co/decapoda-research/llama-13b-hf/resolve/main/pytorch_model-{str(i).rjust(5, '0')}-of-00041.bin"
            for i in range(0, 42)
        ],
    },
    "llama-7b": {
        "config": "https://huggingface.co/huggyllama/llama-7b/resolve/main/config.json",
        "vocab": "https://huggingface.co/huggyllama/llama-7b/resolve/main/tokenizer.model",
        "weight": [
            f"https://huggingface.co/huggyllama/llama-7b/resolve/main/pytorch_model-{str(i).rjust(5, '0')}-of-00002.bin"
            for i in range(1, 3)
        ],
    },
    "llama-13b": {
        "config": "https://huggingface.co/huggyllama/llama-13b/resolve/main/config.json",
        "vocab": "https://huggingface.co/huggyllama/llama-13b/resolve/main/tokenizer.model",
        "weight": [
            f"https://huggingface.co/huggyllama/llama-13b/resolve/main/pytorch_model-{str(i).rjust(5, '0')}-of-00003.bin"
            for i in range(1, 4)
        ],
    },
}

import unitorch.cli.models.llama.modeling
import unitorch.cli.models.llama.processing
from unitorch.cli.models.llama.modeling import (
    LlamaForClassification,
    LlamaForPretrain,
    LlamaForGeneration,
)
from unitorch.cli.models.llama.processing import LlamaProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
import torch.nn as nn
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.loss.prophetnet import ProphetnetLoss
from unitorch.loss.ranking import ListMLELoss, ApproxNDCGLoss, ApproxMRRLoss


class CELoss(nn.Module):
    def __init__(
        self,
        smoothing_alpha: Optional[float] = 0.0,
        weight: Optional[torch.Tensor] = None,
        reduction: Optional[str] = "mean",
    ):
        super().__init__()
        self.smoothing_alpha = smoothing_alpha
        self.weight = weight
        self.reduction = reduction

    def forward(
        self,
        input: torch.Tensor,
        target: torch.Tensor,
        sample_weight: Optional[torch.Tensor] = None,
    ):
        assert input.dim() == 2 and target.dim() == 1

        target = target.long()

        if self.smoothing_alpha > 0:
            batch_size, num_classes = input.size()
            smooth_label = torch.full(
                size=(batch_size, num_classes),
                fill_value=self.smoothing_alpha / (num_classes - 1),
            ).to(loss)
            smooth_label.scatter_(
                dim=1,
                index=torch.unsqueeze(target, dim=1),
                value=(1 - self.smoothing_alpha),
            )
            log_logits = torch.nn.functional.log_softmax(input, dim=1)
            loss = torch.sum(log_logits * smooth_label, dim=1)
        else:
            loss = nn.CrossEntropyLoss(weight=self.weight, reduction="none")(
                input, target
            ).squeeze()

        if sample_weight is not None:
            loss = loss * sample_weight

        if self.reduction == "mean":
            loss = torch.mean(loss)

        return loss


class BCELoss(nn.Module):
    def __init__(
        self,
        weight: Optional[torch.Tensor] = None,
        reduction: Optional[str] = "mean",
    ):
        super().__init__()
        self.weight = weight
        self.reduction = reduction

    def forward(
        self,
        input: torch.Tensor,
        target: torch.Tensor,
        sample_weight: Optional[torch.Tensor] = None,
    ):
        if target.dim() == 1:
            target = torch.unsqueeze(target, dim=1)
        assert input.dim() == 2 and target.dim() == 2

        target = target.float()
        loss = nn.BCEWithLogitsLoss(weight=self.weight, reduction="none")(input, target)

        loss = torch.sum(loss, dim=1)

        if sample_weight is not None:
            loss = loss * sample_weight

        if self.reduction == "mean":
            loss = torch.mean(loss)

        return loss


class LMLoss(nn.Module):
    def __init__(
        self,
        reduction: Optional[str] = "mean",
    ):
        super().__init__()
        self.reduction = reduction

    def forward(
        self,
        input: torch.Tensor,
        target: torch.Tensor,
        masks: Optional[torch.Tensor] = None,
        sample_weight: Optional[torch.Tensor] = None,
    ):
        assert input.dim() == 3 and target.dim() == 2

        batch_size, seq_len, num_classes = input.size()
        input = input.contiguous().view(batch_size * seq_len, num_classes)
        target = target.contiguous().view(-1)
        target = target.long()
        if masks is None:
            masks = torch.ones_like(target).to(target)
        masks = masks.contiguous().view(-1)
        loss = nn.CrossEntropyLoss(reduction="none")(input, target)
        loss = loss * masks.float()
        loss = loss.contiguous().view(batch_size, seq_len).sum(1) / torch.max(
            masks.contiguous().view(batch_size, seq_len).float().sum(1),
            torch.ones(batch_size).to(masks.device),
        )

        if sample_weight is not None:
            loss = loss * sample_weight

        if self.reduction == "mean":
            loss = torch.mean(loss)

        return loss


class MSELoss(nn.Module):
    def __init__(
        self,
        reduction: Optional[str] = "mean",
    ):
        super().__init__()
        self.reduction = reduction

    def forward(
        self,
        input: torch.Tensor,
        target: torch.Tensor,
        sample_weight: Optional[torch.Tensor] = None,
    ):
        assert input.size(0) == target.size(0) and input.numel() == target.numel()
        batch_size = input.size(0)
        loss = nn.MSELoss(reduction="none")(
            input.view(batch_size, -1), target.view(batch_size, -1)
        )

        if loss.dim() > 1:
            loss = torch.sum(loss, dim=1)

        if sample_weight is not None:
            loss = loss * sample_weight

        if self.reduction == "mean":
            loss = torch.mean(loss)

        return loss



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
import torch.nn as nn
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union


class ProphetnetLoss(nn.Module):
    def __init__(
        self,
        reduction: Optional[str] = "mean",
    ):
        super().__init__()
        self.reduction = reduction

    def forward(
        self,
        input: torch.Tensor,
        target: torch.Tensor,
        masks: Optional[torch.Tensor] = None,
        sample_weight: Optional[torch.Tensor] = None,
    ):
        assert input.dim() == 4 and target.dim() == 2

        batch_size, ngram, seq_len, num_classes = input.size()
        input = input.contiguous().view(batch_size * ngram * seq_len, num_classes)
        target = target.repeat(1, ngram).contiguous().view(-1)
        target = target.long()
        if masks is None or masks.numel() == 0:
            masks = torch.ones(batch_size, seq_len).to(target)
        masks = masks.repeat(1, ngram).contiguous().view(-1)

        loss = nn.CrossEntropyLoss(reduction="none")(input, target)
        loss = loss * masks.float()
        loss = loss.contiguous().view(batch_size, ngram * seq_len).sum(1) / torch.max(
            masks.contiguous().view(batch_size, ngram * seq_len).float().sum(1),
            torch.ones(batch_size).to(masks.device),
        )

        if sample_weight is not None and sample_weight.numel() > 0:
            loss = loss * sample_weight

        if self.reduction == "mean":
            loss = torch.mean(loss)

        return loss



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
from functools import partial
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union

import torch
from transformers import XLMProphetNetTokenizer
from unitorch.models import HfTextGenerationProcessor


def get_xprophetnet_tokenizer(
    vocab_path: str,
    special_input_ids: Optional[Dict] = dict(),
):
    assert os.path.exists(vocab_path)
    tokenizer = XLMProphetNetTokenizer(vocab_path)
    for token, _id in special_input_ids.items():
        tokenizer.added_tokens_encoder[token] = _id
        tokenizer.unique_no_split_tokens.append(token)
        tokenizer.added_tokens_decoder[_id] = token
        tokenizer.add_tokens(token, special_tokens=True)
    return tokenizer


class XProphetNetProcessor(HfTextGenerationProcessor):
    def __init__(
        self,
        vocab_path: str,
        special_input_ids: Optional[Dict] = dict(),
        max_seq_length: Optional[int] = 128,
        max_gen_seq_length: Optional[int] = 48,
    ):
        tokenizer = get_xprophetnet_tokenizer(
            vocab_path,
            special_input_ids=special_input_ids,
        )
        super().__init__(
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

# pretrained infos
pretrained_xprophetnet_infos = {
    "default-xprophetnet": {
        "config": "https://huggingface.co/microsoft/xprophetnet-large-wiki100-cased/resolve/main/config.json",
        "vocab": "https://huggingface.co/microsoft/xprophetnet-large-wiki100-cased/resolve/main/prophetnet.tokenizer",
    },
    "xprophetnet-large-wiki100-cased": {
        "config": "https://huggingface.co/microsoft/xprophetnet-large-wiki100-cased/resolve/main/config.json",
        "vocab": "https://huggingface.co/microsoft/xprophetnet-large-wiki100-cased/resolve/main/prophetnet.tokenizer",
        "weight": "https://huggingface.co/microsoft/xprophetnet-large-wiki100-cased/resolve/main/pytorch_model.bin",
    },
}

import unitorch.cli.models.xprophetnet.modeling
import unitorch.cli.models.xprophetnet.processing
from unitorch.cli.models.xprophetnet.modeling import XProphetNetForGeneration
from unitorch.cli.models.xprophetnet.processing import XProphetNetProcessor
